# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: qdrant_service.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'qdrant_service.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import common_pb2 as common__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14qdrant_service.proto\x12\nisa.qdrant\x1a\x0c\x63ommon.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1cgoogle/protobuf/struct.proto\"\x16\n\x06Vector\x12\x0c\n\x04\x64\x61ta\x18\x01 \x03(\x02\"\x8a\x01\n\x0cNamedVectors\x12\x36\n\x07vectors\x18\x01 \x03(\x0b\x32%.isa.qdrant.NamedVectors.VectorsEntry\x1a\x42\n\x0cVectorsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12!\n\x05value\x18\x02 \x01(\x0b\x32\x12.isa.qdrant.Vector:\x02\x38\x01\"\xbf\x01\n\x05Point\x12\x10\n\x06num_id\x18\x01 \x01(\x04H\x00\x12\x10\n\x06str_id\x18\x02 \x01(\tH\x00\x12$\n\x06vector\x18\x03 \x01(\x0b\x32\x12.isa.qdrant.VectorH\x01\x12\x31\n\rnamed_vectors\x18\x04 \x01(\x0b\x32\x18.isa.qdrant.NamedVectorsH\x01\x12(\n\x07payload\x18\x05 \x01(\x0b\x32\x17.google.protobuf.StructB\x04\n\x02idB\t\n\x07vectors\"g\n\x0bScoredPoint\x12 \n\x05point\x18\x01 \x01(\x0b\x32\x11.isa.qdrant.Point\x12\r\n\x05score\x18\x02 \x01(\x02\x12\'\n\x07version\x18\x03 \x01(\x0b\x32\x16.google.protobuf.Value\"\x8f\x01\n\x06\x46ilter\x12)\n\x04must\x18\x01 \x03(\x0b\x32\x1b.isa.qdrant.FilterCondition\x12+\n\x06should\x18\x02 \x03(\x0b\x32\x1b.isa.qdrant.FilterCondition\x12-\n\x08must_not\x18\x03 \x03(\x0b\x32\x1b.isa.qdrant.FilterCondition\"\xb8\x02\n\x0f\x46ilterCondition\x12\r\n\x05\x66ield\x18\x01 \x01(\t\x12+\n\x05match\x18\x02 \x01(\x0b\x32\x1a.isa.qdrant.MatchConditionH\x00\x12+\n\x05range\x18\x03 \x01(\x0b\x32\x1a.isa.qdrant.RangeConditionH\x00\x12?\n\x10geo_bounding_box\x18\x04 \x01(\x0b\x32#.isa.qdrant.GeoBoundingBoxConditionH\x00\x12\x34\n\ngeo_radius\x18\x05 \x01(\x0b\x32\x1e.isa.qdrant.GeoRadiusConditionH\x00\x12\x38\n\x0cvalues_count\x18\x06 \x01(\x0b\x32 .isa.qdrant.ValuesCountConditionH\x00\x42\x0b\n\tcondition\"X\n\x0eMatchCondition\x12\x11\n\x07keyword\x18\x01 \x01(\tH\x00\x12\x11\n\x07integer\x18\x02 \x01(\x03H\x00\x12\x11\n\x07\x62oolean\x18\x03 \x01(\x08H\x00\x42\r\n\x0bmatch_value\"t\n\x0eRangeCondition\x12\x0f\n\x02gt\x18\x01 \x01(\x01H\x00\x88\x01\x01\x12\x10\n\x03gte\x18\x02 \x01(\x01H\x01\x88\x01\x01\x12\x0f\n\x02lt\x18\x03 \x01(\x01H\x02\x88\x01\x01\x12\x10\n\x03lte\x18\x04 \x01(\x01H\x03\x88\x01\x01\x42\x05\n\x03_gtB\x06\n\x04_gteB\x05\n\x03_ltB\x06\n\x04_lte\"m\n\x17GeoBoundingBoxCondition\x12&\n\x08top_left\x18\x01 \x01(\x0b\x32\x14.isa.qdrant.GeoPoint\x12*\n\x0c\x62ottom_right\x18\x02 \x01(\x0b\x32\x14.isa.qdrant.GeoPoint\"Q\n\x12GeoRadiusCondition\x12$\n\x06\x63\x65nter\x18\x01 \x01(\x0b\x32\x14.isa.qdrant.GeoPoint\x12\x15\n\rradius_meters\x18\x02 \x01(\x01\"$\n\x08GeoPoint\x12\x0b\n\x03lat\x18\x01 \x01(\x01\x12\x0b\n\x03lon\x18\x02 \x01(\x01\"z\n\x14ValuesCountCondition\x12\x0f\n\x02lt\x18\x01 \x01(\x04H\x00\x88\x01\x01\x12\x10\n\x03lte\x18\x02 \x01(\x04H\x01\x88\x01\x01\x12\x0f\n\x02gt\x18\x03 \x01(\x04H\x02\x88\x01\x01\x12\x10\n\x03gte\x18\x04 \x01(\x04H\x03\x88\x01\x01\x42\x05\n\x03_ltB\x06\n\x04_lteB\x05\n\x03_gtB\x06\n\x04_gte\"\xe0\x01\n\x0cVectorParams\x12\x0c\n\x04size\x18\x01 \x01(\x04\x12&\n\x08\x64istance\x18\x02 \x01(\x0e\x32\x14.isa.qdrant.Distance\x12\x30\n\x0bhnsw_config\x18\x03 \x01(\x0b\x32\x16.isa.qdrant.HnswConfigH\x00\x88\x01\x01\x12@\n\x13quantization_config\x18\x04 \x01(\x0b\x32\x1e.isa.qdrant.QuantizationConfigH\x01\x88\x01\x01\x42\x0e\n\x0c_hnsw_configB\x16\n\x14_quantization_config\"J\n\nHnswConfig\x12\t\n\x01m\x18\x01 \x01(\x04\x12\x14\n\x0c\x65\x66_construct\x18\x02 \x01(\x04\x12\x1b\n\x13\x66ull_scan_threshold\x18\x03 \x01(\x04\"\x8a\x01\n\x12QuantizationConfig\x12\x30\n\x06scalar\x18\x01 \x01(\x0b\x32\x1e.isa.qdrant.ScalarQuantizationH\x00\x12\x32\n\x07product\x18\x02 \x01(\x0b\x32\x1f.isa.qdrant.ProductQuantizationH\x00\x42\x0e\n\x0cquantization\"\xb1\x01\n\x12ScalarQuantization\x12\x37\n\x04type\x18\x01 \x01(\x0e\x32).isa.qdrant.ScalarQuantization.ScalarType\x12\x15\n\x08quantile\x18\x02 \x01(\x02H\x00\x88\x01\x01\x12\x17\n\nalways_ram\x18\x03 \x01(\x08H\x01\x88\x01\x01\"\x16\n\nScalarType\x12\x08\n\x04INT8\x10\x00\x42\x0b\n\t_quantileB\r\n\x0b_always_ram\"\xc3\x01\n\x13ProductQuantization\x12\x45\n\x0b\x63ompression\x18\x01 \x01(\x0e\x32\x30.isa.qdrant.ProductQuantization.CompressionRatio\x12\x17\n\nalways_ram\x18\x02 \x01(\x08H\x00\x88\x01\x01\"=\n\x10\x43ompressionRatio\x12\x06\n\x02X4\x10\x00\x12\x06\n\x02X8\x10\x01\x12\x07\n\x03X16\x10\x02\x12\x07\n\x03X32\x10\x03\x12\x07\n\x03X64\x10\x04\x42\r\n\x0b_always_ram\"\x9b\x01\n\x13VectorsConfigParams\x12;\n\x06params\x18\x01 \x03(\x0b\x32+.isa.qdrant.VectorsConfigParams.ParamsEntry\x1aG\n\x0bParamsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\'\n\x05value\x18\x02 \x01(\x0b\x32\x18.isa.qdrant.VectorParams:\x02\x38\x01\"\xc0\x03\n\x17\x43reateCollectionRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x17\n\x0f\x63ollection_name\x18\x02 \x01(\t\x12\x31\n\rvector_params\x18\x03 \x01(\x0b\x32\x18.isa.qdrant.VectorParamsH\x00\x12>\n\x13named_vector_params\x18\x04 \x01(\x0b\x32\x1f.isa.qdrant.VectorsConfigParamsH\x00\x12\x19\n\x0cshard_number\x18\x05 \x01(\rH\x01\x88\x01\x01\x12\x1f\n\x12replication_factor\x18\x06 \x01(\rH\x02\x88\x01\x01\x12%\n\x18write_consistency_factor\x18\x07 \x01(\rH\x03\x88\x01\x01\x12\x1c\n\x0fon_disk_payload\x18\x08 \x01(\x08H\x04\x88\x01\x01\x42\x10\n\x0evectors_configB\x0f\n\r_shard_numberB\x15\n\x13_replication_factorB\x1b\n\x19_write_consistency_factorB\x12\n\x10_on_disk_payload\"[\n\x18\x43reateCollectionResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x0f\n\x07success\x18\x02 \x01(\x08\"G\n\x16ListCollectionsRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\"\x81\x01\n\x17ListCollectionsResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x36\n\x0b\x63ollections\x18\x02 \x03(\x0b\x32!.isa.qdrant.CollectionDescription\"%\n\x15\x43ollectionDescription\x12\x0c\n\x04name\x18\x01 \x01(\t\"b\n\x18GetCollectionInfoRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x17\n\x0f\x63ollection_name\x18\x02 \x01(\t\"u\n\x19GetCollectionInfoResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12(\n\x04info\x18\x02 \x01(\x0b\x32\x1a.isa.qdrant.CollectionInfo\"\xaa\x01\n\x0e\x43ollectionInfo\x12\x0e\n\x06status\x18\x01 \x01(\t\x12\x14\n\x0cpoints_count\x18\x02 \x01(\x04\x12\x16\n\x0esegments_count\x18\x03 \x01(\x04\x12\'\n\x06\x63onfig\x18\x04 \x01(\x0b\x32\x17.google.protobuf.Struct\x12\x31\n\x10optimizer_status\x18\x05 \x01(\x0b\x32\x17.google.protobuf.Struct\"\xd0\x01\n\x17UpdateCollectionRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x17\n\x0f\x63ollection_name\x18\x02 \x01(\t\x12\x1e\n\x11optimizers_config\x18\x03 \x01(\rH\x00\x88\x01\x01\x12,\n\x06params\x18\x04 \x01(\x0b\x32\x17.google.protobuf.StructH\x01\x88\x01\x01\x42\x14\n\x12_optimizers_configB\t\n\x07_params\"[\n\x18UpdateCollectionResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x0f\n\x07success\x18\x02 \x01(\x08\"a\n\x17\x44\x65leteCollectionRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x17\n\x0f\x63ollection_name\x18\x02 \x01(\t\"[\n\x18\x44\x65leteCollectionResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x0f\n\x07success\x18\x02 \x01(\x08\"z\n\x1c\x43reateCollectionAliasRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x17\n\x0f\x63ollection_name\x18\x02 \x01(\t\x12\x12\n\nalias_name\x18\x03 \x01(\t\"`\n\x1d\x43reateCollectionAliasResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x0f\n\x07success\x18\x02 \x01(\x08\"\xc0\x01\n\x13UpsertPointsRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x17\n\x0f\x63ollection_name\x18\x02 \x01(\t\x12!\n\x06points\x18\x03 \x03(\x0b\x32\x11.isa.qdrant.Point\x12\x11\n\x04wait\x18\x04 \x01(\x08H\x00\x88\x01\x01\x12\x15\n\x08ordering\x18\x05 \x01(\tH\x01\x88\x01\x01\x42\x07\n\x05_waitB\x0b\n\t_ordering\"l\n\x14UpsertPointsResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x14\n\x0coperation_id\x18\x02 \x01(\t\x12\x0e\n\x06status\x18\x03 \x01(\t\"\xd4\x01\n\x10GetPointsRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x17\n\x0f\x63ollection_name\x18\x02 \x01(\t\x12 \n\x03ids\x18\x03 \x03(\x0b\x32\x13.isa.qdrant.PointId\x12\x19\n\x0cwith_payload\x18\x04 \x01(\x08H\x00\x88\x01\x01\x12\x19\n\x0cwith_vectors\x18\x05 \x01(\x08H\x01\x88\x01\x01\x42\x0f\n\r_with_payloadB\x0f\n\r_with_vectors\"-\n\x07PointId\x12\r\n\x03num\x18\x01 \x01(\x04H\x00\x12\r\n\x03str\x18\x02 \x01(\tH\x00\x42\x04\n\x02id\"f\n\x11GetPointsResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12!\n\x06points\x18\x02 \x03(\x0b\x32\x11.isa.qdrant.Point\"/\n\x0bPointIdList\x12 \n\x03ids\x18\x01 \x03(\x0b\x32\x13.isa.qdrant.PointId\"\xd3\x01\n\x13\x44\x65letePointsRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x17\n\x0f\x63ollection_name\x18\x02 \x01(\t\x12&\n\x03ids\x18\x03 \x01(\x0b\x32\x17.isa.qdrant.PointIdListH\x00\x12$\n\x06\x66ilter\x18\x04 \x01(\x0b\x32\x12.isa.qdrant.FilterH\x00\x12\x11\n\x04wait\x18\x05 \x01(\x08H\x01\x88\x01\x01\x42\n\n\x08selectorB\x07\n\x05_wait\"l\n\x14\x44\x65letePointsResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x14\n\x0coperation_id\x18\x02 \x01(\t\x12\x0e\n\x06status\x18\x03 \x01(\t\"\xfe\x01\n\x14UpdatePayloadRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x17\n\x0f\x63ollection_name\x18\x02 \x01(\t\x12(\n\x07payload\x18\x03 \x01(\x0b\x32\x17.google.protobuf.Struct\x12&\n\x03ids\x18\x04 \x01(\x0b\x32\x17.isa.qdrant.PointIdListH\x00\x12$\n\x06\x66ilter\x18\x05 \x01(\x0b\x32\x12.isa.qdrant.FilterH\x00\x12\x11\n\x04wait\x18\x06 \x01(\x08H\x01\x88\x01\x01\x42\n\n\x08selectorB\x07\n\x05_wait\"m\n\x15UpdatePayloadResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x14\n\x0coperation_id\x18\x02 \x01(\t\x12\x0e\n\x06status\x18\x03 \x01(\t\"\xe2\x01\n\x14\x44\x65letePayloadRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x17\n\x0f\x63ollection_name\x18\x02 \x01(\t\x12\x0c\n\x04keys\x18\x03 \x03(\t\x12&\n\x03ids\x18\x04 \x01(\x0b\x32\x17.isa.qdrant.PointIdListH\x00\x12$\n\x06\x66ilter\x18\x05 \x01(\x0b\x32\x12.isa.qdrant.FilterH\x00\x12\x11\n\x04wait\x18\x06 \x01(\x08H\x01\x88\x01\x01\x42\n\n\x08selectorB\x07\n\x05_wait\"m\n\x15\x44\x65letePayloadResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x14\n\x0coperation_id\x18\x02 \x01(\t\x12\x0e\n\x06status\x18\x03 \x01(\t\"\xd3\x01\n\x13\x43learPayloadRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x17\n\x0f\x63ollection_name\x18\x02 \x01(\t\x12&\n\x03ids\x18\x03 \x01(\x0b\x32\x17.isa.qdrant.PointIdListH\x00\x12$\n\x06\x66ilter\x18\x04 \x01(\x0b\x32\x12.isa.qdrant.FilterH\x00\x12\x11\n\x04wait\x18\x05 \x01(\x08H\x01\x88\x01\x01\x42\n\n\x08selectorB\x07\n\x05_wait\"l\n\x14\x43learPayloadResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x14\n\x0coperation_id\x18\x02 \x01(\t\x12\x0e\n\x06status\x18\x03 \x01(\t\"\xcb\x03\n\rSearchRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x17\n\x0f\x63ollection_name\x18\x02 \x01(\t\x12\"\n\x06vector\x18\x03 \x01(\x0b\x32\x12.isa.qdrant.Vector\x12\x18\n\x0bvector_name\x18\x04 \x01(\tH\x00\x88\x01\x01\x12\'\n\x06\x66ilter\x18\x05 \x01(\x0b\x32\x12.isa.qdrant.FilterH\x01\x88\x01\x01\x12,\n\x06params\x18\x06 \x01(\x0b\x32\x17.google.protobuf.StructH\x02\x88\x01\x01\x12\r\n\x05limit\x18\x07 \x01(\x04\x12\x13\n\x06offset\x18\x08 \x01(\x04H\x03\x88\x01\x01\x12\x19\n\x0cwith_payload\x18\t \x01(\x08H\x04\x88\x01\x01\x12\x19\n\x0cwith_vectors\x18\n \x01(\x08H\x05\x88\x01\x01\x12\x1c\n\x0fscore_threshold\x18\x0b \x01(\x02H\x06\x88\x01\x01\x42\x0e\n\x0c_vector_nameB\t\n\x07_filterB\t\n\x07_paramsB\t\n\x07_offsetB\x0f\n\r_with_payloadB\x0f\n\r_with_vectorsB\x12\n\x10_score_threshold\"i\n\x0eSearchResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\'\n\x06result\x18\x02 \x03(\x0b\x32\x17.isa.qdrant.ScoredPoint\"\x89\x01\n\x12SearchBatchRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x17\n\x0f\x63ollection_name\x18\x02 \x01(\t\x12+\n\x08searches\x18\x03 \x03(\x0b\x32\x19.isa.qdrant.SearchRequest\"r\n\x13SearchBatchResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12+\n\x07results\x18\x02 \x03(\x0b\x32\x1a.isa.qdrant.SearchResponse\"\xb9\x03\n\x10RecommendRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x17\n\x0f\x63ollection_name\x18\x02 \x01(\t\x12%\n\x08positive\x18\x03 \x03(\x0b\x32\x13.isa.qdrant.PointId\x12%\n\x08negative\x18\x04 \x03(\x0b\x32\x13.isa.qdrant.PointId\x12\'\n\x06\x66ilter\x18\x05 \x01(\x0b\x32\x12.isa.qdrant.FilterH\x00\x88\x01\x01\x12\r\n\x05limit\x18\x06 \x01(\x04\x12\x13\n\x06offset\x18\x07 \x01(\x04H\x01\x88\x01\x01\x12\x19\n\x0cwith_payload\x18\x08 \x01(\x08H\x02\x88\x01\x01\x12\x19\n\x0cwith_vectors\x18\t \x01(\x08H\x03\x88\x01\x01\x12\x1c\n\x0fscore_threshold\x18\n \x01(\x02H\x04\x88\x01\x01\x12\x15\n\x08strategy\x18\x0b \x01(\tH\x05\x88\x01\x01\x42\t\n\x07_filterB\t\n\x07_offsetB\x0f\n\r_with_payloadB\x0f\n\r_with_vectorsB\x12\n\x10_score_thresholdB\x0b\n\t_strategy\"l\n\x11RecommendResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\'\n\x06result\x18\x02 \x03(\x0b\x32\x17.isa.qdrant.ScoredPoint\"\xdf\x02\n\x0f\x44iscoverRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x17\n\x0f\x63ollection_name\x18\x02 \x01(\t\x12#\n\x06target\x18\x03 \x01(\x0b\x32\x13.isa.qdrant.PointId\x12$\n\x07\x63ontext\x18\x04 \x03(\x0b\x32\x13.isa.qdrant.PointId\x12\'\n\x06\x66ilter\x18\x05 \x01(\x0b\x32\x12.isa.qdrant.FilterH\x00\x88\x01\x01\x12\r\n\x05limit\x18\x06 \x01(\x04\x12\x13\n\x06offset\x18\x07 \x01(\x04H\x01\x88\x01\x01\x12\x19\n\x0cwith_payload\x18\x08 \x01(\x08H\x02\x88\x01\x01\x12\x19\n\x0cwith_vectors\x18\t \x01(\x08H\x03\x88\x01\x01\x42\t\n\x07_filterB\t\n\x07_offsetB\x0f\n\r_with_payloadB\x0f\n\r_with_vectors\"k\n\x10\x44iscoverResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\'\n\x06result\x18\x02 \x03(\x0b\x32\x17.isa.qdrant.ScoredPoint\"\xb6\x02\n\rScrollRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x17\n\x0f\x63ollection_name\x18\x02 \x01(\t\x12\'\n\x06\x66ilter\x18\x03 \x01(\x0b\x32\x12.isa.qdrant.FilterH\x00\x88\x01\x01\x12\x12\n\x05limit\x18\x04 \x01(\rH\x01\x88\x01\x01\x12(\n\x06offset\x18\x05 \x01(\x0b\x32\x13.isa.qdrant.PointIdH\x02\x88\x01\x01\x12\x19\n\x0cwith_payload\x18\x06 \x01(\x08H\x03\x88\x01\x01\x12\x19\n\x0cwith_vectors\x18\x07 \x01(\x08H\x04\x88\x01\x01\x42\t\n\x07_filterB\x08\n\x06_limitB\t\n\x07_offsetB\x0f\n\r_with_payloadB\x0f\n\r_with_vectors\"\xac\x01\n\x0eScrollResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12!\n\x06points\x18\x02 \x03(\x0b\x32\x11.isa.qdrant.Point\x12\x32\n\x10next_page_offset\x18\x03 \x01(\x0b\x32\x13.isa.qdrant.PointIdH\x00\x88\x01\x01\x42\x13\n\x11_next_page_offset\"\xa8\x01\n\x0c\x43ountRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x17\n\x0f\x63ollection_name\x18\x02 \x01(\t\x12\'\n\x06\x66ilter\x18\x03 \x01(\x0b\x32\x12.isa.qdrant.FilterH\x00\x88\x01\x01\x12\x12\n\x05\x65xact\x18\x04 \x01(\x08H\x01\x88\x01\x01\x42\t\n\x07_filterB\x08\n\x06_exact\"N\n\rCountResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\r\n\x05\x63ount\x18\x02 \x01(\x04\"\x8a\x02\n\x17\x43reateFieldIndexRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x17\n\x0f\x63ollection_name\x18\x02 \x01(\t\x12\x12\n\nfield_name\x18\x03 \x01(\t\x12\x17\n\nfield_type\x18\x04 \x01(\tH\x00\x88\x01\x01\x12\x38\n\x12\x66ield_index_params\x18\x05 \x01(\x0b\x32\x17.google.protobuf.StructH\x01\x88\x01\x01\x12\x11\n\x04wait\x18\x06 \x01(\x08H\x02\x88\x01\x01\x42\r\n\x0b_field_typeB\x15\n\x13_field_index_paramsB\x07\n\x05_wait\"p\n\x18\x43reateFieldIndexResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x14\n\x0coperation_id\x18\x02 \x01(\t\x12\x0e\n\x06status\x18\x03 \x01(\t\"\x91\x01\n\x17\x44\x65leteFieldIndexRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x17\n\x0f\x63ollection_name\x18\x02 \x01(\t\x12\x12\n\nfield_name\x18\x03 \x01(\t\x12\x11\n\x04wait\x18\x04 \x01(\x08H\x00\x88\x01\x01\x42\x07\n\x05_wait\"p\n\x18\x44\x65leteFieldIndexResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x14\n\x0coperation_id\x18\x02 \x01(\t\x12\x0e\n\x06status\x18\x03 \x01(\t\"_\n\x15\x43reateSnapshotRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x17\n\x0f\x63ollection_name\x18\x02 \x01(\t\"\x8f\x01\n\x16\x43reateSnapshotResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x15\n\rsnapshot_name\x18\x02 \x01(\t\x12.\n\ncreated_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"^\n\x14ListSnapshotsRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x17\n\x0f\x63ollection_name\x18\x02 \x01(\t\"{\n\x15ListSnapshotsResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x32\n\tsnapshots\x18\x02 \x03(\x0b\x32\x1f.isa.qdrant.SnapshotDescription\"g\n\x13SnapshotDescription\x12\x0c\n\x04name\x18\x01 \x01(\t\x12.\n\ncreated_at\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x12\n\nsize_bytes\x18\x03 \x01(\x04\"v\n\x15\x44\x65leteSnapshotRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x17\n\x0f\x63ollection_name\x18\x02 \x01(\t\x12\x15\n\rsnapshot_name\x18\x03 \x01(\t\"Y\n\x16\x44\x65leteSnapshotResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x0f\n\x07success\x18\x02 \x01(\x08\"I\n\x18QdrantHealthCheckRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\"m\n\x19QdrantHealthCheckResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x0f\n\x07healthy\x18\x02 \x01(\x08\x12\x0f\n\x07version\x18\x03 \x01(\t\"F\n\x15GetClusterInfoRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\"w\n\x16GetClusterInfoResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12-\n\x0c\x63luster_info\x18\x02 \x01(\x0b\x32\x17.google.protobuf.Struct*t\n\x08\x44istance\x12\x14\n\x10\x44ISTANCE_UNKNOWN\x10\x00\x12\x13\n\x0f\x44ISTANCE_COSINE\x10\x01\x12\x13\n\x0f\x44ISTANCE_EUCLID\x10\x02\x12\x10\n\x0c\x44ISTANCE_DOT\x10\x03\x12\x16\n\x12\x44ISTANCE_MANHATTAN\x10\x04\x32\xe3\x10\n\rQdrantService\x12]\n\x10\x43reateCollection\x12#.isa.qdrant.CreateCollectionRequest\x1a$.isa.qdrant.CreateCollectionResponse\x12Z\n\x0fListCollections\x12\".isa.qdrant.ListCollectionsRequest\x1a#.isa.qdrant.ListCollectionsResponse\x12`\n\x11GetCollectionInfo\x12$.isa.qdrant.GetCollectionInfoRequest\x1a%.isa.qdrant.GetCollectionInfoResponse\x12]\n\x10UpdateCollection\x12#.isa.qdrant.UpdateCollectionRequest\x1a$.isa.qdrant.UpdateCollectionResponse\x12]\n\x10\x44\x65leteCollection\x12#.isa.qdrant.DeleteCollectionRequest\x1a$.isa.qdrant.DeleteCollectionResponse\x12l\n\x15\x43reateCollectionAlias\x12(.isa.qdrant.CreateCollectionAliasRequest\x1a).isa.qdrant.CreateCollectionAliasResponse\x12Q\n\x0cUpsertPoints\x12\x1f.isa.qdrant.UpsertPointsRequest\x1a .isa.qdrant.UpsertPointsResponse\x12H\n\tGetPoints\x12\x1c.isa.qdrant.GetPointsRequest\x1a\x1d.isa.qdrant.GetPointsResponse\x12Q\n\x0c\x44\x65letePoints\x12\x1f.isa.qdrant.DeletePointsRequest\x1a .isa.qdrant.DeletePointsResponse\x12T\n\rUpdatePayload\x12 .isa.qdrant.UpdatePayloadRequest\x1a!.isa.qdrant.UpdatePayloadResponse\x12T\n\rDeletePayload\x12 .isa.qdrant.DeletePayloadRequest\x1a!.isa.qdrant.DeletePayloadResponse\x12Q\n\x0c\x43learPayload\x12\x1f.isa.qdrant.ClearPayloadRequest\x1a .isa.qdrant.ClearPayloadResponse\x12?\n\x06Search\x12\x19.isa.qdrant.SearchRequest\x1a\x1a.isa.qdrant.SearchResponse\x12N\n\x0bSearchBatch\x12\x1e.isa.qdrant.SearchBatchRequest\x1a\x1f.isa.qdrant.SearchBatchResponse\x12H\n\tRecommend\x12\x1c.isa.qdrant.RecommendRequest\x1a\x1d.isa.qdrant.RecommendResponse\x12\x45\n\x08\x44iscover\x12\x1b.isa.qdrant.DiscoverRequest\x1a\x1c.isa.qdrant.DiscoverResponse\x12?\n\x06Scroll\x12\x19.isa.qdrant.ScrollRequest\x1a\x1a.isa.qdrant.ScrollResponse\x12<\n\x05\x43ount\x12\x18.isa.qdrant.CountRequest\x1a\x19.isa.qdrant.CountResponse\x12]\n\x10\x43reateFieldIndex\x12#.isa.qdrant.CreateFieldIndexRequest\x1a$.isa.qdrant.CreateFieldIndexResponse\x12]\n\x10\x44\x65leteFieldIndex\x12#.isa.qdrant.DeleteFieldIndexRequest\x1a$.isa.qdrant.DeleteFieldIndexResponse\x12W\n\x0e\x43reateSnapshot\x12!.isa.qdrant.CreateSnapshotRequest\x1a\".isa.qdrant.CreateSnapshotResponse\x12T\n\rListSnapshots\x12 .isa.qdrant.ListSnapshotsRequest\x1a!.isa.qdrant.ListSnapshotsResponse\x12W\n\x0e\x44\x65leteSnapshot\x12!.isa.qdrant.DeleteSnapshotRequest\x1a\".isa.qdrant.DeleteSnapshotResponse\x12Z\n\x0bHealthCheck\x12$.isa.qdrant.QdrantHealthCheckRequest\x1a%.isa.qdrant.QdrantHealthCheckResponse\x12W\n\x0eGetClusterInfo\x12!.isa.qdrant.GetClusterInfoRequest\x1a\".isa.qdrant.GetClusterInfoResponseB1Z/github.com/isa-cloud/isa_cloud/api/proto/qdrantb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qdrant_service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z/github.com/isa-cloud/isa_cloud/api/proto/qdrant'
  _globals['_NAMEDVECTORS_VECTORSENTRY']._loaded_options = None
  _globals['_NAMEDVECTORS_VECTORSENTRY']._serialized_options = b'8\001'
  _globals['_VECTORSCONFIGPARAMS_PARAMSENTRY']._loaded_options = None
  _globals['_VECTORSCONFIGPARAMS_PARAMSENTRY']._serialized_options = b'8\001'
  _globals['_DISTANCE']._serialized_start=10941
  _globals['_DISTANCE']._serialized_end=11057
  _globals['_VECTOR']._serialized_start=113
  _globals['_VECTOR']._serialized_end=135
  _globals['_NAMEDVECTORS']._serialized_start=138
  _globals['_NAMEDVECTORS']._serialized_end=276
  _globals['_NAMEDVECTORS_VECTORSENTRY']._serialized_start=210
  _globals['_NAMEDVECTORS_VECTORSENTRY']._serialized_end=276
  _globals['_POINT']._serialized_start=279
  _globals['_POINT']._serialized_end=470
  _globals['_SCOREDPOINT']._serialized_start=472
  _globals['_SCOREDPOINT']._serialized_end=575
  _globals['_FILTER']._serialized_start=578
  _globals['_FILTER']._serialized_end=721
  _globals['_FILTERCONDITION']._serialized_start=724
  _globals['_FILTERCONDITION']._serialized_end=1036
  _globals['_MATCHCONDITION']._serialized_start=1038
  _globals['_MATCHCONDITION']._serialized_end=1126
  _globals['_RANGECONDITION']._serialized_start=1128
  _globals['_RANGECONDITION']._serialized_end=1244
  _globals['_GEOBOUNDINGBOXCONDITION']._serialized_start=1246
  _globals['_GEOBOUNDINGBOXCONDITION']._serialized_end=1355
  _globals['_GEORADIUSCONDITION']._serialized_start=1357
  _globals['_GEORADIUSCONDITION']._serialized_end=1438
  _globals['_GEOPOINT']._serialized_start=1440
  _globals['_GEOPOINT']._serialized_end=1476
  _globals['_VALUESCOUNTCONDITION']._serialized_start=1478
  _globals['_VALUESCOUNTCONDITION']._serialized_end=1600
  _globals['_VECTORPARAMS']._serialized_start=1603
  _globals['_VECTORPARAMS']._serialized_end=1827
  _globals['_HNSWCONFIG']._serialized_start=1829
  _globals['_HNSWCONFIG']._serialized_end=1903
  _globals['_QUANTIZATIONCONFIG']._serialized_start=1906
  _globals['_QUANTIZATIONCONFIG']._serialized_end=2044
  _globals['_SCALARQUANTIZATION']._serialized_start=2047
  _globals['_SCALARQUANTIZATION']._serialized_end=2224
  _globals['_SCALARQUANTIZATION_SCALARTYPE']._serialized_start=2174
  _globals['_SCALARQUANTIZATION_SCALARTYPE']._serialized_end=2196
  _globals['_PRODUCTQUANTIZATION']._serialized_start=2227
  _globals['_PRODUCTQUANTIZATION']._serialized_end=2422
  _globals['_PRODUCTQUANTIZATION_COMPRESSIONRATIO']._serialized_start=2346
  _globals['_PRODUCTQUANTIZATION_COMPRESSIONRATIO']._serialized_end=2407
  _globals['_VECTORSCONFIGPARAMS']._serialized_start=2425
  _globals['_VECTORSCONFIGPARAMS']._serialized_end=2580
  _globals['_VECTORSCONFIGPARAMS_PARAMSENTRY']._serialized_start=2509
  _globals['_VECTORSCONFIGPARAMS_PARAMSENTRY']._serialized_end=2580
  _globals['_CREATECOLLECTIONREQUEST']._serialized_start=2583
  _globals['_CREATECOLLECTIONREQUEST']._serialized_end=3031
  _globals['_CREATECOLLECTIONRESPONSE']._serialized_start=3033
  _globals['_CREATECOLLECTIONRESPONSE']._serialized_end=3124
  _globals['_LISTCOLLECTIONSREQUEST']._serialized_start=3126
  _globals['_LISTCOLLECTIONSREQUEST']._serialized_end=3197
  _globals['_LISTCOLLECTIONSRESPONSE']._serialized_start=3200
  _globals['_LISTCOLLECTIONSRESPONSE']._serialized_end=3329
  _globals['_COLLECTIONDESCRIPTION']._serialized_start=3331
  _globals['_COLLECTIONDESCRIPTION']._serialized_end=3368
  _globals['_GETCOLLECTIONINFOREQUEST']._serialized_start=3370
  _globals['_GETCOLLECTIONINFOREQUEST']._serialized_end=3468
  _globals['_GETCOLLECTIONINFORESPONSE']._serialized_start=3470
  _globals['_GETCOLLECTIONINFORESPONSE']._serialized_end=3587
  _globals['_COLLECTIONINFO']._serialized_start=3590
  _globals['_COLLECTIONINFO']._serialized_end=3760
  _globals['_UPDATECOLLECTIONREQUEST']._serialized_start=3763
  _globals['_UPDATECOLLECTIONREQUEST']._serialized_end=3971
  _globals['_UPDATECOLLECTIONRESPONSE']._serialized_start=3973
  _globals['_UPDATECOLLECTIONRESPONSE']._serialized_end=4064
  _globals['_DELETECOLLECTIONREQUEST']._serialized_start=4066
  _globals['_DELETECOLLECTIONREQUEST']._serialized_end=4163
  _globals['_DELETECOLLECTIONRESPONSE']._serialized_start=4165
  _globals['_DELETECOLLECTIONRESPONSE']._serialized_end=4256
  _globals['_CREATECOLLECTIONALIASREQUEST']._serialized_start=4258
  _globals['_CREATECOLLECTIONALIASREQUEST']._serialized_end=4380
  _globals['_CREATECOLLECTIONALIASRESPONSE']._serialized_start=4382
  _globals['_CREATECOLLECTIONALIASRESPONSE']._serialized_end=4478
  _globals['_UPSERTPOINTSREQUEST']._serialized_start=4481
  _globals['_UPSERTPOINTSREQUEST']._serialized_end=4673
  _globals['_UPSERTPOINTSRESPONSE']._serialized_start=4675
  _globals['_UPSERTPOINTSRESPONSE']._serialized_end=4783
  _globals['_GETPOINTSREQUEST']._serialized_start=4786
  _globals['_GETPOINTSREQUEST']._serialized_end=4998
  _globals['_POINTID']._serialized_start=5000
  _globals['_POINTID']._serialized_end=5045
  _globals['_GETPOINTSRESPONSE']._serialized_start=5047
  _globals['_GETPOINTSRESPONSE']._serialized_end=5149
  _globals['_POINTIDLIST']._serialized_start=5151
  _globals['_POINTIDLIST']._serialized_end=5198
  _globals['_DELETEPOINTSREQUEST']._serialized_start=5201
  _globals['_DELETEPOINTSREQUEST']._serialized_end=5412
  _globals['_DELETEPOINTSRESPONSE']._serialized_start=5414
  _globals['_DELETEPOINTSRESPONSE']._serialized_end=5522
  _globals['_UPDATEPAYLOADREQUEST']._serialized_start=5525
  _globals['_UPDATEPAYLOADREQUEST']._serialized_end=5779
  _globals['_UPDATEPAYLOADRESPONSE']._serialized_start=5781
  _globals['_UPDATEPAYLOADRESPONSE']._serialized_end=5890
  _globals['_DELETEPAYLOADREQUEST']._serialized_start=5893
  _globals['_DELETEPAYLOADREQUEST']._serialized_end=6119
  _globals['_DELETEPAYLOADRESPONSE']._serialized_start=6121
  _globals['_DELETEPAYLOADRESPONSE']._serialized_end=6230
  _globals['_CLEARPAYLOADREQUEST']._serialized_start=6233
  _globals['_CLEARPAYLOADREQUEST']._serialized_end=6444
  _globals['_CLEARPAYLOADRESPONSE']._serialized_start=6446
  _globals['_CLEARPAYLOADRESPONSE']._serialized_end=6554
  _globals['_SEARCHREQUEST']._serialized_start=6557
  _globals['_SEARCHREQUEST']._serialized_end=7016
  _globals['_SEARCHRESPONSE']._serialized_start=7018
  _globals['_SEARCHRESPONSE']._serialized_end=7123
  _globals['_SEARCHBATCHREQUEST']._serialized_start=7126
  _globals['_SEARCHBATCHREQUEST']._serialized_end=7263
  _globals['_SEARCHBATCHRESPONSE']._serialized_start=7265
  _globals['_SEARCHBATCHRESPONSE']._serialized_end=7379
  _globals['_RECOMMENDREQUEST']._serialized_start=7382
  _globals['_RECOMMENDREQUEST']._serialized_end=7823
  _globals['_RECOMMENDRESPONSE']._serialized_start=7825
  _globals['_RECOMMENDRESPONSE']._serialized_end=7933
  _globals['_DISCOVERREQUEST']._serialized_start=7936
  _globals['_DISCOVERREQUEST']._serialized_end=8287
  _globals['_DISCOVERRESPONSE']._serialized_start=8289
  _globals['_DISCOVERRESPONSE']._serialized_end=8396
  _globals['_SCROLLREQUEST']._serialized_start=8399
  _globals['_SCROLLREQUEST']._serialized_end=8709
  _globals['_SCROLLRESPONSE']._serialized_start=8712
  _globals['_SCROLLRESPONSE']._serialized_end=8884
  _globals['_COUNTREQUEST']._serialized_start=8887
  _globals['_COUNTREQUEST']._serialized_end=9055
  _globals['_COUNTRESPONSE']._serialized_start=9057
  _globals['_COUNTRESPONSE']._serialized_end=9135
  _globals['_CREATEFIELDINDEXREQUEST']._serialized_start=9138
  _globals['_CREATEFIELDINDEXREQUEST']._serialized_end=9404
  _globals['_CREATEFIELDINDEXRESPONSE']._serialized_start=9406
  _globals['_CREATEFIELDINDEXRESPONSE']._serialized_end=9518
  _globals['_DELETEFIELDINDEXREQUEST']._serialized_start=9521
  _globals['_DELETEFIELDINDEXREQUEST']._serialized_end=9666
  _globals['_DELETEFIELDINDEXRESPONSE']._serialized_start=9668
  _globals['_DELETEFIELDINDEXRESPONSE']._serialized_end=9780
  _globals['_CREATESNAPSHOTREQUEST']._serialized_start=9782
  _globals['_CREATESNAPSHOTREQUEST']._serialized_end=9877
  _globals['_CREATESNAPSHOTRESPONSE']._serialized_start=9880
  _globals['_CREATESNAPSHOTRESPONSE']._serialized_end=10023
  _globals['_LISTSNAPSHOTSREQUEST']._serialized_start=10025
  _globals['_LISTSNAPSHOTSREQUEST']._serialized_end=10119
  _globals['_LISTSNAPSHOTSRESPONSE']._serialized_start=10121
  _globals['_LISTSNAPSHOTSRESPONSE']._serialized_end=10244
  _globals['_SNAPSHOTDESCRIPTION']._serialized_start=10246
  _globals['_SNAPSHOTDESCRIPTION']._serialized_end=10349
  _globals['_DELETESNAPSHOTREQUEST']._serialized_start=10351
  _globals['_DELETESNAPSHOTREQUEST']._serialized_end=10469
  _globals['_DELETESNAPSHOTRESPONSE']._serialized_start=10471
  _globals['_DELETESNAPSHOTRESPONSE']._serialized_end=10560
  _globals['_QDRANTHEALTHCHECKREQUEST']._serialized_start=10562
  _globals['_QDRANTHEALTHCHECKREQUEST']._serialized_end=10635
  _globals['_QDRANTHEALTHCHECKRESPONSE']._serialized_start=10637
  _globals['_QDRANTHEALTHCHECKRESPONSE']._serialized_end=10746
  _globals['_GETCLUSTERINFOREQUEST']._serialized_start=10748
  _globals['_GETCLUSTERINFOREQUEST']._serialized_end=10818
  _globals['_GETCLUSTERINFORESPONSE']._serialized_start=10820
  _globals['_GETCLUSTERINFORESPONSE']._serialized_end=10939
  _globals['_QDRANTSERVICE']._serialized_start=11060
  _globals['_QDRANTSERVICE']._serialized_end=13207
# @@protoc_insertion_point(module_scope)



"""
Billing Event Models for Event-Driven Architecture

All services use NATS to communicate via events instead of direct HTTP calls.
This provides loose coupling, better scalability, and fault tolerance.
"""

from enum import Enum
from typing import Optional, Dict, Any
from datetime import datetime
from decimal import Decimal
from pydantic import BaseModel, Field


class EventType(str, Enum):
    """Event types for billing system"""
    # Usage events (published by isA_Model, isA_Agent, etc.)
    USAGE_RECORDED = "usage.recorded"

    # Billing events (published by billing_service)
    COST_CALCULATED = "billing.calculated"
    BILLING_FAILED = "billing.failed"

    # Wallet events (published by wallet_service)
    TOKENS_DEDUCTED = "wallet.tokens.deducted"
    TOKENS_INSUFFICIENT = "wallet.tokens.insufficient"

    # Product events (published by product_service)
    USAGE_METRICS_RECORDED = "product.usage.recorded"


class UnitType(str, Enum):
    """Unit types for different services"""
    TOKEN = "token"           # LLM tokens (GPT-4, GPT-3.5, embeddings)
    IMAGE = "image"           # Image generation (DALL-E, Stable Diffusion)
    MINUTE = "minute"         # Audio processing (STT, TTS)
    CHARACTER = "character"   # TTS character count
    REQUEST = "request"       # API calls (agent tools, searches)
    BYTE = "byte"            # Storage (Minio, S3)
    SECOND = "second"        # Video processing


# ====================
# Usage Events (Source: isA_Model, isA_Agent)
# ====================

class UsageEvent(BaseModel):
    """
    Published when any service uses a billable resource.

    Publishers: isA_Model, isA_Agent, storage_service, etc.
    Subscribers: billing_service, product_service (for metrics)

    NATS Subject: usage.recorded.{product_id}
    Example: usage.recorded.gpt-4, usage.recorded.dall-e-3
    """
    event_type: str = EventType.USAGE_RECORDED
    event_id: str = Field(default_factory=lambda: f"evt_{datetime.utcnow().timestamp()}")

    # User context
    user_id: str = Field(..., description="User who triggered the usage")
    organization_id: Optional[str] = Field(None, description="Organization context")
    subscription_id: Optional[str] = Field(None, description="Active subscription")

    # Usage details
    product_id: str = Field(..., description="Product being used (gpt-4, dall-e-3, etc)")
    usage_amount: Decimal = Field(..., description="Amount used in native units")
    unit_type: UnitType = Field(..., description="Unit type (token, image, etc)")

    # Session tracking
    session_id: Optional[str] = Field(None, description="User session ID")
    request_id: Optional[str] = Field(None, description="Request trace ID")

    # Metadata
    usage_details: Dict[str, Any] = Field(default_factory=dict, description="Additional context")
    timestamp: datetime = Field(default_factory=datetime.utcnow)

    class Config:
        json_encoders = {
            Decimal: lambda v: float(v),
            datetime: lambda v: v.isoformat()
        }


# ====================
# Billing Events (Source: billing_service)
# ====================

class BillingCalculatedEvent(BaseModel):
    """
    Published after billing_service calculates the cost.

    Publisher: billing_service
    Subscribers: wallet_service (to deduct tokens), analytics_service

    NATS Subject: billing.calculated
    """
    event_type: str = EventType.COST_CALCULATED
    event_id: str = Field(default_factory=lambda: f"evt_{datetime.utcnow().timestamp()}")

    # References
    user_id: str
    billing_record_id: str = Field(..., description="Created billing record ID")
    usage_event_id: str = Field(..., description="Original usage event ID")

    # Product info
    product_id: str
    actual_usage: Decimal = Field(..., description="Original usage amount")
    unit_type: UnitType

    # Cost calculation
    token_equivalent: Decimal = Field(..., description="Normalized to token equivalents")
    cost_usd: Decimal = Field(..., description="Actual USD cost")
    unit_price: Decimal = Field(..., description="Price per unit in USD")

    # Token conversion rate
    token_conversion_rate: Decimal = Field(
        ...,
        description="How many tokens this represents (e.g., 1 image = 1333 tokens)"
    )

    # Billing status
    is_free_tier: bool = False
    is_included_in_subscription: bool = False

    timestamp: datetime = Field(default_factory=datetime.utcnow)

    class Config:
        json_encoders = {
            Decimal: lambda v: float(v),
            datetime: lambda v: v.isoformat()
        }


class BillingErrorEvent(BaseModel):
    """
    Published when billing calculation fails.

    Publisher: billing_service
    Subscribers: notification_service, monitoring_service

    NATS Subject: billing.failed
    """
    event_type: str = EventType.BILLING_FAILED
    event_id: str = Field(default_factory=lambda: f"evt_{datetime.utcnow().timestamp()}")

    user_id: str
    usage_event_id: str
    product_id: str

    error_code: str = Field(..., description="Error code (PRICING_NOT_FOUND, etc)")
    error_message: str
    retry_count: int = 0

    timestamp: datetime = Field(default_factory=datetime.utcnow)

    class Config:
        json_encoders = {
            datetime: lambda v: v.isoformat()
        }


# ====================
# Wallet Events (Source: wallet_service)
# ====================

class TokensDeductedEvent(BaseModel):
    """
    Published after wallet_service successfully deducts tokens.

    Publisher: wallet_service
    Subscribers: analytics_service, notification_service

    NATS Subject: wallet.tokens.deducted
    """
    event_type: str = EventType.TOKENS_DEDUCTED
    event_id: str = Field(default_factory=lambda: f"evt_{datetime.utcnow().timestamp()}")

    # References
    user_id: str
    billing_record_id: str
    transaction_id: str = Field(..., description="Wallet transaction ID")

    # Token info
    tokens_deducted: Decimal
    balance_before: Decimal
    balance_after: Decimal

    # Quota tracking
    monthly_quota: Optional[Decimal] = Field(None, description="Monthly token quota")
    monthly_used: Optional[Decimal] = Field(None, description="Tokens used this month")
    percentage_used: Optional[float] = Field(None, description="% of monthly quota used")

    timestamp: datetime = Field(default_factory=datetime.utcnow)

    class Config:
        json_encoders = {
            Decimal: lambda v: float(v),
            datetime: lambda v: v.isoformat()
        }


class TokensInsufficientEvent(BaseModel):
    """
    Published when user doesn't have enough tokens.

    Publisher: wallet_service
    Subscribers: notification_service (alert user), billing_service (mark failed)

    NATS Subject: wallet.tokens.insufficient
    """
    event_type: str = EventType.TOKENS_INSUFFICIENT
    event_id: str = Field(default_factory=lambda: f"evt_{datetime.utcnow().timestamp()}")

    user_id: str
    billing_record_id: str

    tokens_required: Decimal
    tokens_available: Decimal
    tokens_deficit: Decimal

    suggested_action: str = "upgrade_plan"  # or "purchase_tokens"

    timestamp: datetime = Field(default_factory=datetime.utcnow)

    class Config:
        json_encoders = {
            Decimal: lambda v: float(v),
            datetime: lambda v: v.isoformat()
        }


# ====================
# Helper Functions
# ====================

def create_usage_event(
    user_id: str,
    product_id: str,
    usage_amount: Decimal,
    unit_type: UnitType,
    **kwargs
) -> UsageEvent:
    """
    Helper to create a usage event.

    Usage:
        event = create_usage_event(
            user_id="user_123",
            product_id="gpt-4",
            usage_amount=Decimal("1500"),
            unit_type=UnitType.TOKEN,
            session_id="session_abc"
        )
    """
    return UsageEvent(
        user_id=user_id,
        product_id=product_id,
        usage_amount=usage_amount,
        unit_type=unit_type,
        **kwargs
    )


def get_nats_subject(event: BaseModel) -> str:
    """
    Get the NATS subject for an event.

    Returns:
        - usage.recorded.{product_id} for UsageEvent
        - billing.calculated for BillingCalculatedEvent
        - wallet.tokens.deducted for TokensDeductedEvent
        - etc.
    """
    if isinstance(event, UsageEvent):
        return f"usage.recorded.{event.product_id}"
    elif isinstance(event, BillingCalculatedEvent):
        return "billing.calculated"
    elif isinstance(event, TokensDeductedEvent):
        return "wallet.tokens.deducted"
    elif isinstance(event, TokensInsufficientEvent):
        return "wallet.tokens.insufficient"
    elif isinstance(event, BillingErrorEvent):
        return "billing.failed"
    else:
        return "unknown.event"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
NATS Event Publisher for Billing System

Provides helper functions to publish billing events to NATS message bus.
"""

import json
import logging
from typing import Optional, Dict, Any, TYPE_CHECKING
from decimal import Decimal
from datetime import datetime

from ..nats_client import NATSClient
from .billing_events import (
    UsageEvent,
    BillingCalculatedEvent,
    TokensDeductedEvent,
    TokensInsufficientEvent,
    BillingErrorEvent,
    get_nats_subject
)

if TYPE_CHECKING:
    from ..consul_client import ConsulRegistry

logger = logging.getLogger(__name__)


class BillingEventPublisher:
    """
    Publishes billing events to NATS message bus.

    Usage:
        publisher = BillingEventPublisher(nats_host='localhost', nats_port=50056)

        # Publish usage event
        await publisher.publish_usage(
            user_id="user_123",
            product_id="mcp-tool-web-search",
            usage_amount=1,
            unit_type="request",
            usage_details={"model_cost_usd": 0.0015, "tokens": 500}
        )
    """

    def __init__(
        self,
        nats_host: Optional[str] = None,
        nats_port: Optional[int] = None,
        user_id: Optional[str] = None,
        organization_id: Optional[str] = None,
        enable_compression: bool = False,
        consul_registry: Optional['ConsulRegistry'] = None
    ):
        """
        Initialize event publisher.

        Args:
            nats_host: NATS service host (optional, will use Consul discovery if not provided)
            nats_port: NATS service gRPC port (optional, will use Consul discovery if not provided)
            user_id: Default user ID for events
            organization_id: Default organization ID
            enable_compression: Enable message compression
            consul_registry: ConsulRegistry instance for service discovery (optional)
        """
        self.nats_client = NATSClient(
            host=nats_host,
            port=nats_port,
            user_id=user_id,
            organization_id=organization_id,
            enable_compression=enable_compression,
            consul_registry=consul_registry
        )
        self.default_user_id = user_id
        self.default_org_id = organization_id
        self.consul_registry = consul_registry

    def __enter__(self):
        """Context manager entry"""
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """Context manager exit"""
        self.close()

    async def __aenter__(self):
        """Async context manager entry"""
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        """Async context manager exit"""
        await self.aclose()

    def close(self):
        """Close NATS connection"""
        if hasattr(self.nats_client, 'close'):
            self.nats_client.close()

    async def aclose(self):
        """Async close NATS connection"""
        if hasattr(self.nats_client, 'aclose'):
            await self.nats_client.aclose()
        elif hasattr(self.nats_client, 'close'):
            self.nats_client.close()

    def _serialize_event(self, event) -> bytes:
        """
        Serialize Pydantic event to JSON bytes.

        Args:
            event: Pydantic event model

        Returns:
            JSON bytes ready for NATS
        """
        # Use Pydantic's JSON serialization with custom encoders
        json_str = event.model_dump_json()
        return json_str.encode('utf-8')

    async def publish_usage(
        self,
        user_id: str,
        product_id: str,
        usage_amount: Decimal,
        unit_type: str,
        organization_id: Optional[str] = None,
        subscription_id: Optional[str] = None,
        session_id: Optional[str] = None,
        request_id: Optional[str] = None,
        usage_details: Optional[Dict[str, Any]] = None
    ) -> bool:
        """
        Publish a usage event.

        Args:
            user_id: User who triggered the usage
            product_id: Product being used (e.g., "mcp-tool-web-search", "gpt-4")
            usage_amount: Amount used in native units
            unit_type: Unit type (token, image, minute, request, etc.)
            organization_id: Organization context
            subscription_id: Active subscription
            session_id: User session ID
            request_id: Request trace ID
            usage_details: Additional context (model costs, metadata, etc.)

        Returns:
            True if published successfully, False otherwise

        Example:
            await publisher.publish_usage(
                user_id="user_123",
                product_id="mcp-tool-web-search",
                usage_amount=Decimal("1"),
                unit_type="request",
                usage_details={
                    "model_cost_usd": 0.0015,
                    "model_tokens": 500,
                    "model_product": "gpt-4",
                    "tool_result_size": 1024
                }
            )
        """
        try:
            event = UsageEvent(
                user_id=user_id,
                product_id=product_id,
                usage_amount=usage_amount,
                unit_type=unit_type,
                organization_id=organization_id or self.default_org_id,
                subscription_id=subscription_id,
                session_id=session_id,
                request_id=request_id,
                usage_details=usage_details or {}
            )

            subject = get_nats_subject(event)
            data = self._serialize_event(event)

            result = self.nats_client.publish(
                subject=subject,
                data=data,
                headers={"event_type": "usage.recorded"}
            )

            if result and result.get('success'):
                logger.info(f"Published usage event: {product_id} for user {user_id}")
                return True
            else:
                logger.error(f"Failed to publish usage event: {result}")
                return False

        except Exception as e:
            logger.error(f"Error publishing usage event: {e}", exc_info=True)
            return False

    async def publish_billing_calculated(
        self,
        user_id: str,
        billing_record_id: str,
        usage_event_id: str,
        product_id: str,
        actual_usage: Decimal,
        unit_type: str,
        token_equivalent: Decimal,
        cost_usd: Decimal,
        unit_price: Decimal,
        token_conversion_rate: Decimal,
        is_free_tier: bool = False,
        is_included_in_subscription: bool = False
    ) -> bool:
        """
        Publish a billing calculated event.

        Args:
            user_id: User ID
            billing_record_id: Created billing record ID
            usage_event_id: Original usage event ID
            product_id: Product ID
            actual_usage: Original usage amount
            unit_type: Unit type
            token_equivalent: Normalized to token equivalents
            cost_usd: Actual USD cost
            unit_price: Price per unit in USD
            token_conversion_rate: How many tokens this represents
            is_free_tier: Whether this is free tier usage
            is_included_in_subscription: Whether included in subscription

        Returns:
            True if published successfully
        """
        try:
            event = BillingCalculatedEvent(
                user_id=user_id,
                billing_record_id=billing_record_id,
                usage_event_id=usage_event_id,
                product_id=product_id,
                actual_usage=actual_usage,
                unit_type=unit_type,
                token_equivalent=token_equivalent,
                cost_usd=cost_usd,
                unit_price=unit_price,
                token_conversion_rate=token_conversion_rate,
                is_free_tier=is_free_tier,
                is_included_in_subscription=is_included_in_subscription
            )

            subject = get_nats_subject(event)
            data = self._serialize_event(event)

            result = self.nats_client.publish(
                subject=subject,
                data=data,
                headers={"event_type": "billing.calculated"}
            )

            if result and result.get('success'):
                logger.info(f"Published billing calculated event: {billing_record_id}")
                return True
            else:
                logger.error(f"Failed to publish billing calculated event: {result}")
                return False

        except Exception as e:
            logger.error(f"Error publishing billing calculated event: {e}", exc_info=True)
            return False

    async def publish_tokens_deducted(
        self,
        user_id: str,
        billing_record_id: str,
        transaction_id: str,
        tokens_deducted: Decimal,
        balance_before: Decimal,
        balance_after: Decimal,
        monthly_quota: Optional[Decimal] = None,
        monthly_used: Optional[Decimal] = None,
        percentage_used: Optional[float] = None
    ) -> bool:
        """
        Publish a tokens deducted event.

        Args:
            user_id: User ID
            billing_record_id: Billing record ID
            transaction_id: Wallet transaction ID
            tokens_deducted: Tokens deducted
            balance_before: Balance before deduction
            balance_after: Balance after deduction
            monthly_quota: Monthly token quota
            monthly_used: Tokens used this month
            percentage_used: % of monthly quota used

        Returns:
            True if published successfully
        """
        try:
            event = TokensDeductedEvent(
                user_id=user_id,
                billing_record_id=billing_record_id,
                transaction_id=transaction_id,
                tokens_deducted=tokens_deducted,
                balance_before=balance_before,
                balance_after=balance_after,
                monthly_quota=monthly_quota,
                monthly_used=monthly_used,
                percentage_used=percentage_used
            )

            subject = get_nats_subject(event)
            data = self._serialize_event(event)

            result = self.nats_client.publish(
                subject=subject,
                data=data,
                headers={"event_type": "wallet.tokens.deducted"}
            )

            if result and result.get('success'):
                logger.info(f"Published tokens deducted event: {transaction_id}")
                return True
            else:
                logger.error(f"Failed to publish tokens deducted event: {result}")
                return False

        except Exception as e:
            logger.error(f"Error publishing tokens deducted event: {e}", exc_info=True)
            return False

    async def publish_tokens_insufficient(
        self,
        user_id: str,
        billing_record_id: str,
        tokens_required: Decimal,
        tokens_available: Decimal,
        suggested_action: str = "upgrade_plan"
    ) -> bool:
        """
        Publish a tokens insufficient event.

        Args:
            user_id: User ID
            billing_record_id: Billing record ID
            tokens_required: Tokens required
            tokens_available: Tokens available
            suggested_action: Suggested action (upgrade_plan, purchase_tokens)

        Returns:
            True if published successfully
        """
        try:
            tokens_deficit = tokens_required - tokens_available

            event = TokensInsufficientEvent(
                user_id=user_id,
                billing_record_id=billing_record_id,
                tokens_required=tokens_required,
                tokens_available=tokens_available,
                tokens_deficit=tokens_deficit,
                suggested_action=suggested_action
            )

            subject = get_nats_subject(event)
            data = self._serialize_event(event)

            result = self.nats_client.publish(
                subject=subject,
                data=data,
                headers={"event_type": "wallet.tokens.insufficient"}
            )

            if result and result.get('success'):
                logger.info(f"Published tokens insufficient event for user {user_id}")
                return True
            else:
                logger.error(f"Failed to publish tokens insufficient event: {result}")
                return False

        except Exception as e:
            logger.error(f"Error publishing tokens insufficient event: {e}", exc_info=True)
            return False

    async def publish_billing_error(
        self,
        user_id: str,
        usage_event_id: str,
        product_id: str,
        error_code: str,
        error_message: str,
        retry_count: int = 0
    ) -> bool:
        """
        Publish a billing error event.

        Args:
            user_id: User ID
            usage_event_id: Usage event ID
            product_id: Product ID
            error_code: Error code (PRICING_NOT_FOUND, etc.)
            error_message: Error message
            retry_count: Retry count

        Returns:
            True if published successfully
        """
        try:
            event = BillingErrorEvent(
                user_id=user_id,
                usage_event_id=usage_event_id,
                product_id=product_id,
                error_code=error_code,
                error_message=error_message,
                retry_count=retry_count
            )

            subject = get_nats_subject(event)
            data = self._serialize_event(event)

            result = self.nats_client.publish(
                subject=subject,
                data=data,
                headers={"event_type": "billing.failed"}
            )

            if result and result.get('success'):
                logger.warning(f"Published billing error event: {error_code}")
                return True
            else:
                logger.error(f"Failed to publish billing error event: {result}")
                return False

        except Exception as e:
            logger.error(f"Error publishing billing error event: {e}", exc_info=True)
            return False


# Convenience function for quick usage event publishing
async def publish_usage_event(
    user_id: str,
    product_id: str,
    usage_amount: Decimal,
    unit_type: str,
    nats_host: str = 'localhost',
    nats_port: int = 50056,
    **kwargs
) -> bool:
    """
    Quick helper to publish a usage event.

    Example:
        from core.events.event_publisher import publish_usage_event

        success = await publish_usage_event(
            user_id="user_123",
            product_id="mcp-tool-web-search",
            usage_amount=Decimal("1"),
            unit_type="request",
            usage_details={"model_cost": 0.0015}
        )
    """
    async with BillingEventPublisher(
        nats_host=nats_host,
        nats_port=nats_port,
        user_id=user_id
    ) as publisher:
        return await publisher.publish_usage(
            user_id=user_id,
            product_id=product_id,
            usage_amount=usage_amount,
            unit_type=unit_type,
            **kwargs
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: nats_service.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'nats_service.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import common_pb2 as common__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12nats_service.proto\x12\x08isa.nats\x1a\x0c\x63ommon.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/duration.proto\"\xd2\x01\n\x0bNATSMessage\x12\x0f\n\x07subject\x18\x01 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\x12\x33\n\x07headers\x18\x03 \x03(\x0b\x32\".isa.nats.NATSMessage.HeadersEntry\x12\x10\n\x08reply_to\x18\x04 \x01(\t\x12-\n\ttimestamp\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x1a.\n\x0cHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xf0\x01\n\x0cStreamConfig\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08subjects\x18\x02 \x03(\t\x12&\n\x07storage\x18\x03 \x01(\x0e\x32\x15.isa.nats.StorageType\x12\x10\n\x08max_msgs\x18\x04 \x01(\x05\x12\x11\n\tmax_bytes\x18\x05 \x01(\x03\x12*\n\x07max_age\x18\x06 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x14\n\x0cmax_msg_size\x18\x07 \x01(\x05\x12\x10\n\x08replicas\x18\x08 \x01(\x05\x12\x1f\n\x17\x64iscard_new_per_subject\x18\t \x01(\x08\"\x95\x01\n\nStreamInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\x12&\n\x06\x63onfig\x18\x02 \x01(\x0b\x32\x16.isa.nats.StreamConfig\x12$\n\x05state\x18\x03 \x01(\x0b\x32\x15.isa.nats.StreamState\x12+\n\x07\x63reated\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\xdc\x01\n\x0bStreamState\x12\x10\n\x08messages\x18\x01 \x01(\x03\x12\r\n\x05\x62ytes\x18\x02 \x01(\x03\x12\x11\n\tfirst_seq\x18\x03 \x01(\x03\x12\x10\n\x08last_seq\x18\x04 \x01(\x03\x12,\n\x08\x66irst_ts\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12+\n\x07last_ts\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x14\n\x0cnum_subjects\x18\x07 \x01(\x05\x12\x16\n\x0e\x63onsumer_count\x18\x08 \x01(\x05\"\xe4\x02\n\x0e\x43onsumerConfig\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x14\n\x0c\x64urable_name\x18\x02 \x01(\t\x12\x16\n\x0e\x66ilter_subject\x18\x03 \x01(\t\x12\x31\n\x0f\x64\x65livery_policy\x18\x04 \x01(\x0e\x32\x18.isa.nats.DeliveryPolicy\x12\'\n\nack_policy\x18\x05 \x01(\x0e\x32\x13.isa.nats.AckPolicy\x12+\n\x08\x61\x63k_wait\x18\x06 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x13\n\x0bmax_deliver\x18\x07 \x01(\x05\x12-\n\rreplay_policy\x18\x08 \x01(\x0e\x32\x16.isa.nats.ReplayPolicy\x12\x15\n\ropt_start_seq\x18\t \x01(\x03\x12\x32\n\x0eopt_start_time\x18\n \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\xb0\x01\n\x0c\x43onsumerInfo\x12\x13\n\x0bstream_name\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12(\n\x06\x63onfig\x18\x03 \x01(\x0b\x32\x18.isa.nats.ConsumerConfig\x12&\n\x05state\x18\x04 \x01(\x0b\x32\x17.isa.nats.ConsumerState\x12+\n\x07\x63reated\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"k\n\rConsumerState\x12\x15\n\rdelivered_seq\x18\x01 \x01(\x03\x12\x15\n\rack_floor_seq\x18\x02 \x01(\x03\x12\x13\n\x0bnum_pending\x18\x03 \x01(\x03\x12\x17\n\x0fnum_redelivered\x18\x04 \x01(\x03\"\xd3\x01\n\x0ePublishRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x0f\n\x07subject\x18\x03 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x04 \x01(\x0c\x12\x36\n\x07headers\x18\x05 \x03(\x0b\x32%.isa.nats.PublishRequest.HeadersEntry\x12\x10\n\x08reply_to\x18\x06 \x01(\t\x1a.\n\x0cHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"3\n\x0fPublishResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\"h\n\x13PublishBatchRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\'\n\x08messages\x18\x03 \x03(\x0b\x32\x15.isa.nats.NATSMessage\"P\n\x14PublishBatchResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x17\n\x0fpublished_count\x18\x02 \x01(\x05\x12\x0e\n\x06\x65rrors\x18\x03 \x03(\t\"b\n\x10SubscribeRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x0f\n\x07subject\x18\x03 \x01(\t\x12\x13\n\x0bqueue_group\x18\x04 \x01(\t\"\xec\x01\n\x0fMessageResponse\x12\x0f\n\x07subject\x18\x01 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\x12\x37\n\x07headers\x18\x03 \x03(\x0b\x32&.isa.nats.MessageResponse.HeadersEntry\x12\x10\n\x08reply_to\x18\x04 \x01(\t\x12-\n\ttimestamp\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x10\n\x08sequence\x18\x06 \x01(\x03\x1a.\n\x0cHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"O\n\x12UnsubscribeRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x0f\n\x07subject\x18\x03 \x01(\t\"&\n\x13UnsubscribeResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\"\x85\x01\n\x0eRequestRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x0f\n\x07subject\x18\x03 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x04 \x01(\x0c\x12*\n\x07timeout\x18\x05 \x01(\x0b\x32\x19.google.protobuf.Duration\"G\n\x0fRequestResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\x12\x15\n\rerror_message\x18\x03 \x01(\t\"g\n\x15QueueSubscribeRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x0f\n\x07subject\x18\x03 \x01(\t\x12\x13\n\x0bqueue_group\x18\x04 \x01(\t\"g\n\x13\x43reateStreamRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12&\n\x06\x63onfig\x18\x03 \x01(\x0b\x32\x16.isa.nats.StreamConfig\"^\n\x14\x43reateStreamResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12$\n\x06stream\x18\x02 \x01(\x0b\x32\x14.isa.nats.StreamInfo\x12\x0f\n\x07message\x18\x03 \x01(\t\"T\n\x13\x44\x65leteStreamRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x13\n\x0bstream_name\x18\x03 \x01(\t\"\'\n\x14\x44\x65leteStreamResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\"U\n\x14GetStreamInfoRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x13\n\x0bstream_name\x18\x03 \x01(\t\"=\n\x15GetStreamInfoResponse\x12$\n\x06stream\x18\x01 \x01(\x0b\x32\x14.isa.nats.StreamInfo\">\n\x12ListStreamsRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\"<\n\x13ListStreamsResponse\x12%\n\x07streams\x18\x01 \x03(\x0b\x32\x14.isa.nats.StreamInfo\"|\n\x13UpdateStreamRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x13\n\x0bstream_name\x18\x03 \x01(\t\x12&\n\x06\x63onfig\x18\x04 \x01(\x0b\x32\x16.isa.nats.StreamConfig\"M\n\x14UpdateStreamResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12$\n\x06stream\x18\x02 \x01(\x0b\x32\x14.isa.nats.StreamInfo\"d\n\x12PurgeStreamRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x13\n\x0bstream_name\x18\x03 \x01(\t\x12\x0f\n\x07subject\x18\x04 \x01(\t\"<\n\x13PurgeStreamResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x14\n\x0cpurged_count\x18\x02 \x01(\x03\"\x80\x01\n\x15\x43reateConsumerRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x13\n\x0bstream_name\x18\x03 \x01(\t\x12(\n\x06\x63onfig\x18\x04 \x01(\x0b\x32\x18.isa.nats.ConsumerConfig\"d\n\x16\x43reateConsumerResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12(\n\x08\x63onsumer\x18\x02 \x01(\x0b\x32\x16.isa.nats.ConsumerInfo\x12\x0f\n\x07message\x18\x03 \x01(\t\"m\n\x15\x44\x65leteConsumerRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x13\n\x0bstream_name\x18\x03 \x01(\t\x12\x15\n\rconsumer_name\x18\x04 \x01(\t\")\n\x16\x44\x65leteConsumerResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\"n\n\x16GetConsumerInfoRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x13\n\x0bstream_name\x18\x03 \x01(\t\x12\x15\n\rconsumer_name\x18\x04 \x01(\t\"C\n\x17GetConsumerInfoResponse\x12(\n\x08\x63onsumer\x18\x01 \x01(\x0b\x32\x16.isa.nats.ConsumerInfo\"U\n\x14ListConsumersRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x13\n\x0bstream_name\x18\x03 \x01(\t\"B\n\x15ListConsumersResponse\x12)\n\tconsumers\x18\x01 \x03(\x0b\x32\x16.isa.nats.ConsumerInfo\"\xe6\x01\n\x16PublishToStreamRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x13\n\x0bstream_name\x18\x03 \x01(\t\x12\x0f\n\x07subject\x18\x04 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x05 \x01(\x0c\x12>\n\x07headers\x18\x06 \x03(\x0b\x32-.isa.nats.PublishToStreamRequest.HeadersEntry\x1a.\n\x0cHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"|\n\x17PublishToStreamResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x10\n\x08sequence\x18\x02 \x01(\x03\x12-\n\ttimestamp\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0f\n\x07message\x18\x04 \x01(\t\"\xac\x01\n\x13PullMessagesRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x13\n\x0bstream_name\x18\x03 \x01(\t\x12\x15\n\rconsumer_name\x18\x04 \x01(\t\x12\x12\n\nbatch_size\x18\x05 \x01(\x05\x12+\n\x08max_wait\x18\x06 \x01(\x0b\x32\x19.google.protobuf.Duration\"D\n\x14PullMessagesResponse\x12,\n\x08messages\x18\x01 \x03(\x0b\x32\x1a.isa.nats.JetStreamMessage\"\xf3\x01\n\x10JetStreamMessage\x12\x0f\n\x07subject\x18\x01 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\x12\x38\n\x07headers\x18\x03 \x03(\x0b\x32\'.isa.nats.JetStreamMessage.HeadersEntry\x12\x10\n\x08sequence\x18\x04 \x01(\x03\x12-\n\ttimestamp\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x15\n\rnum_delivered\x18\x06 \x01(\x05\x1a.\n\x0cHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"{\n\x11\x41\x63kMessageRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x13\n\x0bstream_name\x18\x03 \x01(\t\x12\x15\n\rconsumer_name\x18\x04 \x01(\t\x12\x10\n\x08sequence\x18\x05 \x01(\x03\"%\n\x12\x41\x63kMessageResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\"\xa5\x01\n\x11NakMessageRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x13\n\x0bstream_name\x18\x03 \x01(\t\x12\x15\n\rconsumer_name\x18\x04 \x01(\t\x12\x10\n\x08sequence\x18\x05 \x01(\x03\x12(\n\x05\x64\x65lay\x18\x06 \x01(\x0b\x32\x19.google.protobuf.Duration\"%\n\x12NakMessageResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\"d\n\x0cKVPutRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x0e\n\x06\x62ucket\x18\x03 \x01(\t\x12\x0b\n\x03key\x18\x04 \x01(\t\x12\r\n\x05value\x18\x05 \x01(\x0c\"2\n\rKVPutResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x10\n\x08revision\x18\x02 \x01(\x03\"U\n\x0cKVGetRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x0e\n\x06\x62ucket\x18\x03 \x01(\t\x12\x0b\n\x03key\x18\x04 \x01(\t\"l\n\rKVGetResponse\x12\r\n\x05\x66ound\x18\x01 \x01(\x08\x12\r\n\x05value\x18\x02 \x01(\x0c\x12\x10\n\x08revision\x18\x03 \x01(\x03\x12+\n\x07\x63reated\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"X\n\x0fKVDeleteRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x0e\n\x06\x62ucket\x18\x03 \x01(\t\x12\x0b\n\x03key\x18\x04 \x01(\t\"#\n\x10KVDeleteResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\"I\n\rKVKeysRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x0e\n\x06\x62ucket\x18\x03 \x01(\t\"\x1e\n\x0eKVKeysResponse\x12\x0c\n\x04keys\x18\x01 \x03(\t\"\xdc\x01\n\x10ObjectPutRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x0e\n\x06\x62ucket\x18\x03 \x01(\t\x12\x13\n\x0bobject_name\x18\x04 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x05 \x01(\x0c\x12:\n\x08metadata\x18\x06 \x03(\x0b\x32(.isa.nats.ObjectPutRequest.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"7\n\x11ObjectPutResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x11\n\tobject_id\x18\x02 \x01(\t\"a\n\x10ObjectGetRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x0e\n\x06\x62ucket\x18\x03 \x01(\t\x12\x13\n\x0bobject_name\x18\x04 \x01(\t\"\x9e\x01\n\x11ObjectGetResponse\x12\r\n\x05\x66ound\x18\x01 \x01(\x08\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\x12;\n\x08metadata\x18\x03 \x03(\x0b\x32).isa.nats.ObjectGetResponse.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"d\n\x13ObjectDeleteRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x0e\n\x06\x62ucket\x18\x03 \x01(\t\x12\x13\n\x0bobject_name\x18\x04 \x01(\t\"\'\n\x14ObjectDeleteResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\"M\n\x11ObjectListRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x0e\n\x06\x62ucket\x18\x03 \x01(\t\";\n\x12ObjectListResponse\x12%\n\x07objects\x18\x01 \x03(\x0b\x32\x14.isa.nats.ObjectInfo\"\xbd\x01\n\nObjectInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04size\x18\x02 \x01(\x03\x12,\n\x08modified\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x34\n\x08metadata\x18\x04 \x03(\x0b\x32\".isa.nats.ObjectInfo.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"@\n\x14GetStatisticsRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\"\xd1\x01\n\x15GetStatisticsResponse\x12\x15\n\rtotal_streams\x18\x01 \x01(\x03\x12\x17\n\x0ftotal_consumers\x18\x02 \x01(\x03\x12\x16\n\x0etotal_messages\x18\x03 \x01(\x03\x12\x13\n\x0btotal_bytes\x18\x04 \x01(\x03\x12\x13\n\x0b\x63onnections\x18\x05 \x01(\x05\x12\x0f\n\x07in_msgs\x18\x06 \x01(\x03\x12\x10\n\x08out_msgs\x18\x07 \x01(\x03\x12\x10\n\x08in_bytes\x18\x08 \x01(\x03\x12\x11\n\tout_bytes\x18\t \x01(\x03\"V\n\x15GetStreamStatsRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x13\n\x0bstream_name\x18\x03 \x01(\t\"\x8f\x01\n\x16GetStreamStatsResponse\x12\x13\n\x0bstream_name\x18\x01 \x01(\t\x12\x10\n\x08messages\x18\x02 \x01(\x03\x12\r\n\x05\x62ytes\x18\x03 \x01(\x03\x12\x16\n\x0e\x63onsumer_count\x18\x04 \x01(\x05\x12\x14\n\x0cmessage_rate\x18\x05 \x01(\x01\x12\x11\n\tbyte_rate\x18\x06 \x01(\x01\",\n\x16NATSHealthCheckRequest\x12\x12\n\ndeep_check\x18\x01 \x01(\x08\"\xb0\x01\n\x17NATSHealthCheckResponse\x12\x0f\n\x07healthy\x18\x01 \x01(\x08\x12\x13\n\x0bnats_status\x18\x02 \x01(\t\x12\x19\n\x11jetstream_enabled\x18\x03 \x01(\x08\x12\x13\n\x0b\x63onnections\x18\x04 \x01(\x05\x12.\n\nchecked_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0f\n\x07message\x18\x06 \x01(\t*\xa2\x01\n\x0e\x44\x65liveryPolicy\x12\x10\n\x0c\x44\x45LIVERY_ALL\x10\x00\x12\x11\n\rDELIVERY_LAST\x10\x01\x12\x10\n\x0c\x44\x45LIVERY_NEW\x10\x02\x12\x1e\n\x1a\x44\x45LIVERY_BY_START_SEQUENCE\x10\x03\x12\x1a\n\x16\x44\x45LIVERY_BY_START_TIME\x10\x04\x12\x1d\n\x19\x44\x45LIVERY_LAST_PER_SUBJECT\x10\x05*8\n\tAckPolicy\x12\x10\n\x0c\x41\x43K_EXPLICIT\x10\x00\x12\x0b\n\x07\x41\x43K_ALL\x10\x01\x12\x0c\n\x08\x41\x43K_NONE\x10\x02*7\n\x0cReplayPolicy\x12\x12\n\x0eREPLAY_INSTANT\x10\x00\x12\x13\n\x0fREPLAY_ORIGINAL\x10\x01*3\n\x0bStorageType\x12\x10\n\x0cSTORAGE_FILE\x10\x00\x12\x12\n\x0eSTORAGE_MEMORY\x10\x01\x32\xb5\x12\n\x0bNATSService\x12>\n\x07Publish\x12\x18.isa.nats.PublishRequest\x1a\x19.isa.nats.PublishResponse\x12M\n\x0cPublishBatch\x12\x1d.isa.nats.PublishBatchRequest\x1a\x1e.isa.nats.PublishBatchResponse\x12\x44\n\tSubscribe\x12\x1a.isa.nats.SubscribeRequest\x1a\x19.isa.nats.MessageResponse0\x01\x12J\n\x0bUnsubscribe\x12\x1c.isa.nats.UnsubscribeRequest\x1a\x1d.isa.nats.UnsubscribeResponse\x12>\n\x07Request\x12\x18.isa.nats.RequestRequest\x1a\x19.isa.nats.RequestResponse\x12N\n\x0eQueueSubscribe\x12\x1f.isa.nats.QueueSubscribeRequest\x1a\x19.isa.nats.MessageResponse0\x01\x12M\n\x0c\x43reateStream\x12\x1d.isa.nats.CreateStreamRequest\x1a\x1e.isa.nats.CreateStreamResponse\x12M\n\x0c\x44\x65leteStream\x12\x1d.isa.nats.DeleteStreamRequest\x1a\x1e.isa.nats.DeleteStreamResponse\x12P\n\rGetStreamInfo\x12\x1e.isa.nats.GetStreamInfoRequest\x1a\x1f.isa.nats.GetStreamInfoResponse\x12J\n\x0bListStreams\x12\x1c.isa.nats.ListStreamsRequest\x1a\x1d.isa.nats.ListStreamsResponse\x12M\n\x0cUpdateStream\x12\x1d.isa.nats.UpdateStreamRequest\x1a\x1e.isa.nats.UpdateStreamResponse\x12J\n\x0bPurgeStream\x12\x1c.isa.nats.PurgeStreamRequest\x1a\x1d.isa.nats.PurgeStreamResponse\x12S\n\x0e\x43reateConsumer\x12\x1f.isa.nats.CreateConsumerRequest\x1a .isa.nats.CreateConsumerResponse\x12S\n\x0e\x44\x65leteConsumer\x12\x1f.isa.nats.DeleteConsumerRequest\x1a .isa.nats.DeleteConsumerResponse\x12V\n\x0fGetConsumerInfo\x12 .isa.nats.GetConsumerInfoRequest\x1a!.isa.nats.GetConsumerInfoResponse\x12P\n\rListConsumers\x12\x1e.isa.nats.ListConsumersRequest\x1a\x1f.isa.nats.ListConsumersResponse\x12V\n\x0fPublishToStream\x12 .isa.nats.PublishToStreamRequest\x1a!.isa.nats.PublishToStreamResponse\x12M\n\x0cPullMessages\x12\x1d.isa.nats.PullMessagesRequest\x1a\x1e.isa.nats.PullMessagesResponse\x12G\n\nAckMessage\x12\x1b.isa.nats.AckMessageRequest\x1a\x1c.isa.nats.AckMessageResponse\x12G\n\nNakMessage\x12\x1b.isa.nats.NakMessageRequest\x1a\x1c.isa.nats.NakMessageResponse\x12\x38\n\x05KVPut\x12\x16.isa.nats.KVPutRequest\x1a\x17.isa.nats.KVPutResponse\x12\x38\n\x05KVGet\x12\x16.isa.nats.KVGetRequest\x1a\x17.isa.nats.KVGetResponse\x12\x41\n\x08KVDelete\x12\x19.isa.nats.KVDeleteRequest\x1a\x1a.isa.nats.KVDeleteResponse\x12;\n\x06KVKeys\x12\x17.isa.nats.KVKeysRequest\x1a\x18.isa.nats.KVKeysResponse\x12\x44\n\tObjectPut\x12\x1a.isa.nats.ObjectPutRequest\x1a\x1b.isa.nats.ObjectPutResponse\x12\x44\n\tObjectGet\x12\x1a.isa.nats.ObjectGetRequest\x1a\x1b.isa.nats.ObjectGetResponse\x12M\n\x0cObjectDelete\x12\x1d.isa.nats.ObjectDeleteRequest\x1a\x1e.isa.nats.ObjectDeleteResponse\x12G\n\nObjectList\x12\x1b.isa.nats.ObjectListRequest\x1a\x1c.isa.nats.ObjectListResponse\x12P\n\rGetStatistics\x12\x1e.isa.nats.GetStatisticsRequest\x1a\x1f.isa.nats.GetStatisticsResponse\x12S\n\x0eGetStreamStats\x12\x1f.isa.nats.GetStreamStatsRequest\x1a .isa.nats.GetStreamStatsResponse\x12R\n\x0bHealthCheck\x12 .isa.nats.NATSHealthCheckRequest\x1a!.isa.nats.NATSHealthCheckResponseB/Z-github.com/isa-cloud/isa_cloud/api/proto/natsb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nats_service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z-github.com/isa-cloud/isa_cloud/api/proto/nats'
  _globals['_NATSMESSAGE_HEADERSENTRY']._loaded_options = None
  _globals['_NATSMESSAGE_HEADERSENTRY']._serialized_options = b'8\001'
  _globals['_PUBLISHREQUEST_HEADERSENTRY']._loaded_options = None
  _globals['_PUBLISHREQUEST_HEADERSENTRY']._serialized_options = b'8\001'
  _globals['_MESSAGERESPONSE_HEADERSENTRY']._loaded_options = None
  _globals['_MESSAGERESPONSE_HEADERSENTRY']._serialized_options = b'8\001'
  _globals['_PUBLISHTOSTREAMREQUEST_HEADERSENTRY']._loaded_options = None
  _globals['_PUBLISHTOSTREAMREQUEST_HEADERSENTRY']._serialized_options = b'8\001'
  _globals['_JETSTREAMMESSAGE_HEADERSENTRY']._loaded_options = None
  _globals['_JETSTREAMMESSAGE_HEADERSENTRY']._serialized_options = b'8\001'
  _globals['_OBJECTPUTREQUEST_METADATAENTRY']._loaded_options = None
  _globals['_OBJECTPUTREQUEST_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_OBJECTGETRESPONSE_METADATAENTRY']._loaded_options = None
  _globals['_OBJECTGETRESPONSE_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_OBJECTINFO_METADATAENTRY']._loaded_options = None
  _globals['_OBJECTINFO_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_DELIVERYPOLICY']._serialized_start=8073
  _globals['_DELIVERYPOLICY']._serialized_end=8235
  _globals['_ACKPOLICY']._serialized_start=8237
  _globals['_ACKPOLICY']._serialized_end=8293
  _globals['_REPLAYPOLICY']._serialized_start=8295
  _globals['_REPLAYPOLICY']._serialized_end=8350
  _globals['_STORAGETYPE']._serialized_start=8352
  _globals['_STORAGETYPE']._serialized_end=8403
  _globals['_NATSMESSAGE']._serialized_start=112
  _globals['_NATSMESSAGE']._serialized_end=322
  _globals['_NATSMESSAGE_HEADERSENTRY']._serialized_start=276
  _globals['_NATSMESSAGE_HEADERSENTRY']._serialized_end=322
  _globals['_STREAMCONFIG']._serialized_start=325
  _globals['_STREAMCONFIG']._serialized_end=565
  _globals['_STREAMINFO']._serialized_start=568
  _globals['_STREAMINFO']._serialized_end=717
  _globals['_STREAMSTATE']._serialized_start=720
  _globals['_STREAMSTATE']._serialized_end=940
  _globals['_CONSUMERCONFIG']._serialized_start=943
  _globals['_CONSUMERCONFIG']._serialized_end=1299
  _globals['_CONSUMERINFO']._serialized_start=1302
  _globals['_CONSUMERINFO']._serialized_end=1478
  _globals['_CONSUMERSTATE']._serialized_start=1480
  _globals['_CONSUMERSTATE']._serialized_end=1587
  _globals['_PUBLISHREQUEST']._serialized_start=1590
  _globals['_PUBLISHREQUEST']._serialized_end=1801
  _globals['_PUBLISHREQUEST_HEADERSENTRY']._serialized_start=276
  _globals['_PUBLISHREQUEST_HEADERSENTRY']._serialized_end=322
  _globals['_PUBLISHRESPONSE']._serialized_start=1803
  _globals['_PUBLISHRESPONSE']._serialized_end=1854
  _globals['_PUBLISHBATCHREQUEST']._serialized_start=1856
  _globals['_PUBLISHBATCHREQUEST']._serialized_end=1960
  _globals['_PUBLISHBATCHRESPONSE']._serialized_start=1962
  _globals['_PUBLISHBATCHRESPONSE']._serialized_end=2042
  _globals['_SUBSCRIBEREQUEST']._serialized_start=2044
  _globals['_SUBSCRIBEREQUEST']._serialized_end=2142
  _globals['_MESSAGERESPONSE']._serialized_start=2145
  _globals['_MESSAGERESPONSE']._serialized_end=2381
  _globals['_MESSAGERESPONSE_HEADERSENTRY']._serialized_start=276
  _globals['_MESSAGERESPONSE_HEADERSENTRY']._serialized_end=322
  _globals['_UNSUBSCRIBEREQUEST']._serialized_start=2383
  _globals['_UNSUBSCRIBEREQUEST']._serialized_end=2462
  _globals['_UNSUBSCRIBERESPONSE']._serialized_start=2464
  _globals['_UNSUBSCRIBERESPONSE']._serialized_end=2502
  _globals['_REQUESTREQUEST']._serialized_start=2505
  _globals['_REQUESTREQUEST']._serialized_end=2638
  _globals['_REQUESTRESPONSE']._serialized_start=2640
  _globals['_REQUESTRESPONSE']._serialized_end=2711
  _globals['_QUEUESUBSCRIBEREQUEST']._serialized_start=2713
  _globals['_QUEUESUBSCRIBEREQUEST']._serialized_end=2816
  _globals['_CREATESTREAMREQUEST']._serialized_start=2818
  _globals['_CREATESTREAMREQUEST']._serialized_end=2921
  _globals['_CREATESTREAMRESPONSE']._serialized_start=2923
  _globals['_CREATESTREAMRESPONSE']._serialized_end=3017
  _globals['_DELETESTREAMREQUEST']._serialized_start=3019
  _globals['_DELETESTREAMREQUEST']._serialized_end=3103
  _globals['_DELETESTREAMRESPONSE']._serialized_start=3105
  _globals['_DELETESTREAMRESPONSE']._serialized_end=3144
  _globals['_GETSTREAMINFOREQUEST']._serialized_start=3146
  _globals['_GETSTREAMINFOREQUEST']._serialized_end=3231
  _globals['_GETSTREAMINFORESPONSE']._serialized_start=3233
  _globals['_GETSTREAMINFORESPONSE']._serialized_end=3294
  _globals['_LISTSTREAMSREQUEST']._serialized_start=3296
  _globals['_LISTSTREAMSREQUEST']._serialized_end=3358
  _globals['_LISTSTREAMSRESPONSE']._serialized_start=3360
  _globals['_LISTSTREAMSRESPONSE']._serialized_end=3420
  _globals['_UPDATESTREAMREQUEST']._serialized_start=3422
  _globals['_UPDATESTREAMREQUEST']._serialized_end=3546
  _globals['_UPDATESTREAMRESPONSE']._serialized_start=3548
  _globals['_UPDATESTREAMRESPONSE']._serialized_end=3625
  _globals['_PURGESTREAMREQUEST']._serialized_start=3627
  _globals['_PURGESTREAMREQUEST']._serialized_end=3727
  _globals['_PURGESTREAMRESPONSE']._serialized_start=3729
  _globals['_PURGESTREAMRESPONSE']._serialized_end=3789
  _globals['_CREATECONSUMERREQUEST']._serialized_start=3792
  _globals['_CREATECONSUMERREQUEST']._serialized_end=3920
  _globals['_CREATECONSUMERRESPONSE']._serialized_start=3922
  _globals['_CREATECONSUMERRESPONSE']._serialized_end=4022
  _globals['_DELETECONSUMERREQUEST']._serialized_start=4024
  _globals['_DELETECONSUMERREQUEST']._serialized_end=4133
  _globals['_DELETECONSUMERRESPONSE']._serialized_start=4135
  _globals['_DELETECONSUMERRESPONSE']._serialized_end=4176
  _globals['_GETCONSUMERINFOREQUEST']._serialized_start=4178
  _globals['_GETCONSUMERINFOREQUEST']._serialized_end=4288
  _globals['_GETCONSUMERINFORESPONSE']._serialized_start=4290
  _globals['_GETCONSUMERINFORESPONSE']._serialized_end=4357
  _globals['_LISTCONSUMERSREQUEST']._serialized_start=4359
  _globals['_LISTCONSUMERSREQUEST']._serialized_end=4444
  _globals['_LISTCONSUMERSRESPONSE']._serialized_start=4446
  _globals['_LISTCONSUMERSRESPONSE']._serialized_end=4512
  _globals['_PUBLISHTOSTREAMREQUEST']._serialized_start=4515
  _globals['_PUBLISHTOSTREAMREQUEST']._serialized_end=4745
  _globals['_PUBLISHTOSTREAMREQUEST_HEADERSENTRY']._serialized_start=276
  _globals['_PUBLISHTOSTREAMREQUEST_HEADERSENTRY']._serialized_end=322
  _globals['_PUBLISHTOSTREAMRESPONSE']._serialized_start=4747
  _globals['_PUBLISHTOSTREAMRESPONSE']._serialized_end=4871
  _globals['_PULLMESSAGESREQUEST']._serialized_start=4874
  _globals['_PULLMESSAGESREQUEST']._serialized_end=5046
  _globals['_PULLMESSAGESRESPONSE']._serialized_start=5048
  _globals['_PULLMESSAGESRESPONSE']._serialized_end=5116
  _globals['_JETSTREAMMESSAGE']._serialized_start=5119
  _globals['_JETSTREAMMESSAGE']._serialized_end=5362
  _globals['_JETSTREAMMESSAGE_HEADERSENTRY']._serialized_start=276
  _globals['_JETSTREAMMESSAGE_HEADERSENTRY']._serialized_end=322
  _globals['_ACKMESSAGEREQUEST']._serialized_start=5364
  _globals['_ACKMESSAGEREQUEST']._serialized_end=5487
  _globals['_ACKMESSAGERESPONSE']._serialized_start=5489
  _globals['_ACKMESSAGERESPONSE']._serialized_end=5526
  _globals['_NAKMESSAGEREQUEST']._serialized_start=5529
  _globals['_NAKMESSAGEREQUEST']._serialized_end=5694
  _globals['_NAKMESSAGERESPONSE']._serialized_start=5696
  _globals['_NAKMESSAGERESPONSE']._serialized_end=5733
  _globals['_KVPUTREQUEST']._serialized_start=5735
  _globals['_KVPUTREQUEST']._serialized_end=5835
  _globals['_KVPUTRESPONSE']._serialized_start=5837
  _globals['_KVPUTRESPONSE']._serialized_end=5887
  _globals['_KVGETREQUEST']._serialized_start=5889
  _globals['_KVGETREQUEST']._serialized_end=5974
  _globals['_KVGETRESPONSE']._serialized_start=5976
  _globals['_KVGETRESPONSE']._serialized_end=6084
  _globals['_KVDELETEREQUEST']._serialized_start=6086
  _globals['_KVDELETEREQUEST']._serialized_end=6174
  _globals['_KVDELETERESPONSE']._serialized_start=6176
  _globals['_KVDELETERESPONSE']._serialized_end=6211
  _globals['_KVKEYSREQUEST']._serialized_start=6213
  _globals['_KVKEYSREQUEST']._serialized_end=6286
  _globals['_KVKEYSRESPONSE']._serialized_start=6288
  _globals['_KVKEYSRESPONSE']._serialized_end=6318
  _globals['_OBJECTPUTREQUEST']._serialized_start=6321
  _globals['_OBJECTPUTREQUEST']._serialized_end=6541
  _globals['_OBJECTPUTREQUEST_METADATAENTRY']._serialized_start=6494
  _globals['_OBJECTPUTREQUEST_METADATAENTRY']._serialized_end=6541
  _globals['_OBJECTPUTRESPONSE']._serialized_start=6543
  _globals['_OBJECTPUTRESPONSE']._serialized_end=6598
  _globals['_OBJECTGETREQUEST']._serialized_start=6600
  _globals['_OBJECTGETREQUEST']._serialized_end=6697
  _globals['_OBJECTGETRESPONSE']._serialized_start=6700
  _globals['_OBJECTGETRESPONSE']._serialized_end=6858
  _globals['_OBJECTGETRESPONSE_METADATAENTRY']._serialized_start=6494
  _globals['_OBJECTGETRESPONSE_METADATAENTRY']._serialized_end=6541
  _globals['_OBJECTDELETEREQUEST']._serialized_start=6860
  _globals['_OBJECTDELETEREQUEST']._serialized_end=6960
  _globals['_OBJECTDELETERESPONSE']._serialized_start=6962
  _globals['_OBJECTDELETERESPONSE']._serialized_end=7001
  _globals['_OBJECTLISTREQUEST']._serialized_start=7003
  _globals['_OBJECTLISTREQUEST']._serialized_end=7080
  _globals['_OBJECTLISTRESPONSE']._serialized_start=7082
  _globals['_OBJECTLISTRESPONSE']._serialized_end=7141
  _globals['_OBJECTINFO']._serialized_start=7144
  _globals['_OBJECTINFO']._serialized_end=7333
  _globals['_OBJECTINFO_METADATAENTRY']._serialized_start=6494
  _globals['_OBJECTINFO_METADATAENTRY']._serialized_end=6541
  _globals['_GETSTATISTICSREQUEST']._serialized_start=7335
  _globals['_GETSTATISTICSREQUEST']._serialized_end=7399
  _globals['_GETSTATISTICSRESPONSE']._serialized_start=7402
  _globals['_GETSTATISTICSRESPONSE']._serialized_end=7611
  _globals['_GETSTREAMSTATSREQUEST']._serialized_start=7613
  _globals['_GETSTREAMSTATSREQUEST']._serialized_end=7699
  _globals['_GETSTREAMSTATSRESPONSE']._serialized_start=7702
  _globals['_GETSTREAMSTATSRESPONSE']._serialized_end=7845
  _globals['_NATSHEALTHCHECKREQUEST']._serialized_start=7847
  _globals['_NATSHEALTHCHECKREQUEST']._serialized_end=7891
  _globals['_NATSHEALTHCHECKRESPONSE']._serialized_start=7894
  _globals['_NATSHEALTHCHECKRESPONSE']._serialized_end=8070
  _globals['_NATSSERVICE']._serialized_start=8406
  _globals['_NATSSERVICE']._serialized_end=10763
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python3
"""
Async PostgreSQL gRPC Client
High-performance async PostgreSQL client using grpc.aio

Performance Benefits:
- True async I/O without GIL blocking
- Concurrent query execution
- Memory-efficient connection pooling
"""

from typing import List, Dict, Optional, Any, TYPE_CHECKING
from google.protobuf.struct_pb2 import Struct, Value
from .async_base_client import AsyncBaseGRPCClient
from .proto import postgres_service_pb2, postgres_service_pb2_grpc

if TYPE_CHECKING:
    from .consul_client import ConsulRegistry


class AsyncPostgresClient(AsyncBaseGRPCClient):
    """Async PostgreSQL gRPC client for high-performance concurrent operations."""

    def __init__(
        self,
        host: Optional[str] = None,
        port: Optional[int] = None,
        user_id: Optional[str] = None,
        lazy_connect: bool = True,
        enable_compression: bool = True,
        enable_retry: bool = True,
        consul_registry: Optional['ConsulRegistry'] = None,
        service_name_override: Optional[str] = None
    ):
        """
        Initialize async PostgreSQL client.

        Args:
            host: Service address (optional, will use Consul discovery if not provided)
            port: Service port (optional, will use Consul discovery if not provided)
            user_id: User ID
            lazy_connect: Lazy connection (default: True)
            enable_compression: Enable compression (default: True)
            enable_retry: Enable retry (default: True)
            consul_registry: ConsulRegistry instance for service discovery (optional)
            service_name_override: Override service name for Consul lookup (optional)
        """
        super().__init__(
            host=host,
            port=port,
            user_id=user_id,
            lazy_connect=lazy_connect,
            enable_compression=enable_compression,
            enable_retry=enable_retry,
            consul_registry=consul_registry,
            service_name_override=service_name_override
        )

    def _create_stub(self):
        """Create PostgreSQL service stub."""
        return postgres_service_pb2_grpc.PostgresServiceStub(self.channel)

    def service_name(self) -> str:
        return "PostgreSQL"

    def default_port(self) -> int:
        return 50061

    # ============================================
    # Health Check
    # ============================================

    async def health_check(self, detailed: bool = True) -> Optional[Dict]:
        """Health check."""
        try:
            await self._ensure_connected()
            request = postgres_service_pb2.HealthCheckRequest(detailed=detailed)
            response = await self.stub.HealthCheck(request)

            return {
                'status': response.status,
                'healthy': response.healthy,
                'version': response.version,
                'details': dict(response.details) if response.details else {}
            }

        except Exception as e:
            return self.handle_error(e, "Health check")

    # ============================================
    # Query Operations
    # ============================================

    async def query(self, sql: str, params: Optional[List[Any]] = None, schema: str = 'public') -> Optional[List[Dict]]:
        """Execute SELECT query.

        Args:
            sql: SQL query statement
            params: Query parameters
            schema: Database schema (default: public)

        Returns:
            List of result rows as dictionaries
        """
        try:
            await self._ensure_connected()

            proto_params = []
            if params:
                for param in params:
                    proto_params.append(self._python_to_proto_value(param))

            request = postgres_service_pb2.QueryRequest(
                sql=sql,
                params=proto_params,
                schema=schema
            )

            response = await self.stub.Query(request)

            if response.metadata.success:
                rows = [self._proto_struct_to_dict(row) for row in response.rows]
                return rows
            return None

        except Exception as e:
            return self.handle_error(e, "Query")

    async def query_row(self, sql: str, params: Optional[List[Any]] = None, schema: str = 'public') -> Optional[Dict]:
        """Execute single row query.

        Args:
            sql: SQL query statement
            params: Query parameters
            schema: Database schema

        Returns:
            Single row as dictionary or None
        """
        try:
            await self._ensure_connected()

            proto_params = []
            if params:
                for param in params:
                    proto_params.append(self._python_to_proto_value(param))

            request = postgres_service_pb2.QueryRowRequest(
                sql=sql,
                params=proto_params,
                schema=schema
            )

            response = await self.stub.QueryRow(request)

            if response.metadata.success and response.found:
                return self._proto_struct_to_dict(response.row)
            return None

        except Exception as e:
            return self.handle_error(e, "Query row")

    async def execute(self, sql: str, params: Optional[List[Any]] = None, schema: str = 'public') -> Optional[int]:
        """Execute INSERT/UPDATE/DELETE statement.

        Args:
            sql: SQL statement
            params: Statement parameters
            schema: Database schema

        Returns:
            Number of rows affected
        """
        try:
            await self._ensure_connected()

            proto_params = []
            if params:
                for param in params:
                    proto_params.append(self._python_to_proto_value(param))

            request = postgres_service_pb2.ExecuteRequest(
                sql=sql,
                params=proto_params,
                schema=schema
            )

            response = await self.stub.Execute(request)

            if response.metadata.success:
                return response.rows_affected
            return None

        except Exception as e:
            return self.handle_error(e, "Execute")

    async def execute_batch(self, operations: List[Dict[str, Any]], schema: str = 'public') -> Optional[Dict]:
        """Execute batch operations.

        Args:
            operations: List of {'sql': str, 'params': List} dictionaries
            schema: Database schema

        Returns:
            Batch execution results
        """
        try:
            await self._ensure_connected()

            batch_ops = []
            for op in operations:
                proto_params = []
                if 'params' in op and op['params']:
                    for param in op['params']:
                        proto_params.append(self._python_to_proto_value(param))

                batch_ops.append(postgres_service_pb2.BatchOperation(
                    sql=op['sql'],
                    params=proto_params
                ))

            request = postgres_service_pb2.ExecuteBatchRequest(
                operations=batch_ops,
                schema=schema
            )

            response = await self.stub.ExecuteBatch(request)

            if response.metadata.success:
                return {
                    'total_rows_affected': response.total_rows_affected,
                    'results': [{'rows_affected': r.rows_affected, 'error': r.error} for r in response.results]
                }
            return None

        except Exception as e:
            return self.handle_error(e, "Execute batch")

    # ============================================
    # Query Builder Operations
    # ============================================

    async def select_from(
        self,
        table: str,
        columns: Optional[List[str]] = None,
        where: Optional[List[Dict]] = None,
        order_by: Optional[List[str]] = None,
        limit: int = 0,
        offset: int = 0,
        schema: str = 'public'
    ) -> Optional[List[Dict]]:
        """Query builder style SELECT.

        Args:
            table: Table name
            columns: Columns to select (default: all)
            where: WHERE conditions as list of dicts
            order_by: ORDER BY clauses
            limit: LIMIT value
            offset: OFFSET value
            schema: Database schema

        Returns:
            List of result rows
        """
        try:
            await self._ensure_connected()

            where_clauses = []
            if where:
                for w in where:
                    where_clauses.append(postgres_service_pb2.WhereClause(
                        column=w.get('column', ''),
                        operator=w.get('operator', '='),
                        value=self._python_to_proto_value(w.get('value'))
                    ))

            request = postgres_service_pb2.SelectFromRequest(
                table=table,
                columns=columns or [],
                where=where_clauses,
                order_by=order_by or [],
                limit=limit,
                offset=offset,
                schema=schema
            )

            response = await self.stub.SelectFrom(request)

            if response.metadata.success:
                return [self._proto_struct_to_dict(row) for row in response.rows]
            return None

        except Exception as e:
            return self.handle_error(e, "Select from")

    async def insert_into(
        self,
        table: str,
        rows: List[Dict],
        returning: bool = False,
        schema: str = 'public'
    ) -> Optional[int]:
        """Insert rows into table.

        Args:
            table: Table name
            rows: List of row dictionaries to insert
            returning: Return inserted rows
            schema: Database schema

        Returns:
            Number of rows inserted
        """
        try:
            await self._ensure_connected()

            proto_rows = []
            for row in rows:
                struct = Struct()
                struct.update(row)
                proto_rows.append(struct)

            request = postgres_service_pb2.InsertIntoRequest(
                table=table,
                rows=proto_rows,
                returning=returning,
                schema=schema
            )

            response = await self.stub.InsertInto(request)

            if response.metadata.success:
                return response.rows_inserted
            return None

        except Exception as e:
            return self.handle_error(e, "Insert into")

    # ============================================
    # Table Operations
    # ============================================

    async def list_tables(self, schema: str = 'public') -> List[str]:
        """List all tables in schema.

        Args:
            schema: Database schema

        Returns:
            List of table names
        """
        try:
            await self._ensure_connected()

            request = postgres_service_pb2.ListTablesRequest(schema=schema)
            response = await self.stub.ListTables(request)

            if response.metadata.success:
                return list(response.tables)
            return []

        except Exception as e:
            self.handle_error(e, "List tables")
            return []

    async def table_exists(self, table: str, schema: str = 'public') -> bool:
        """Check if table exists.

        Args:
            table: Table name
            schema: Database schema

        Returns:
            True if table exists, False otherwise
        """
        try:
            await self._ensure_connected()

            request = postgres_service_pb2.TableExistsRequest(
                table=table,
                schema=schema
            )
            response = await self.stub.TableExists(request)

            if response.metadata.success:
                return response.exists
            return False

        except Exception as e:
            self.handle_error(e, "Table exists check")
            return False

    # ============================================
    # Statistics
    # ============================================

    async def get_stats(self) -> Optional[Dict]:
        """Get connection pool and database statistics.

        Returns:
            Statistics dictionary
        """
        try:
            await self._ensure_connected()

            request = postgres_service_pb2.GetStatsRequest()
            response = await self.stub.GetStats(request)

            if response.metadata.success:
                return {
                    'pool': {
                        'max_connections': response.pool_stats.max_connections,
                        'open_connections': response.pool_stats.open_connections,
                        'idle_connections': response.pool_stats.idle_connections,
                        'active_connections': response.pool_stats.active_connections,
                        'total_queries': response.pool_stats.total_queries,
                    },
                    'database': {
                        'version': response.db_stats.version,
                    }
                }
            return None

        except Exception as e:
            return self.handle_error(e, "Get stats")

    # ============================================
    # Helper Methods
    # ============================================

    def _python_to_proto_value(self, value: Any) -> Value:
        """Convert Python value to proto Value."""
        proto_value = Value()

        if value is None:
            proto_value.null_value = 0
        elif isinstance(value, bool):
            proto_value.bool_value = value
        elif isinstance(value, (int, float)):
            proto_value.number_value = float(value)
        elif isinstance(value, str):
            proto_value.string_value = value
        elif isinstance(value, list):
            proto_value.list_value.values.extend([self._python_to_proto_value(v) for v in value])
        elif isinstance(value, dict):
            for k, v in value.items():
                proto_value.struct_value.fields[k].CopyFrom(self._python_to_proto_value(v))
        else:
            proto_value.string_value = str(value)

        return proto_value

    # ============================================
    # Concurrent Operations
    # ============================================

    async def query_many_concurrent(self, queries: List[Dict[str, Any]]) -> List[Optional[List[Dict]]]:
        """
        Execute multiple queries concurrently.

        Args:
            queries: List of {'sql': str, 'params': list, 'schema': str} dicts

        Returns:
            List of results for each query
        """
        import asyncio

        async def execute_query(q: Dict) -> Optional[List[Dict]]:
            return await self.query(
                sql=q.get('sql', ''),
                params=q.get('params'),
                schema=q.get('schema', 'public')
            )

        return await asyncio.gather(*[execute_query(q) for q in queries])

    async def execute_many_concurrent(self, statements: List[Dict[str, Any]]) -> List[Optional[int]]:
        """
        Execute multiple statements concurrently.

        Args:
            statements: List of {'sql': str, 'params': list, 'schema': str} dicts

        Returns:
            List of rows affected for each statement
        """
        import asyncio

        async def execute_stmt(s: Dict) -> Optional[int]:
            return await self.execute(
                sql=s.get('sql', ''),
                params=s.get('params'),
                schema=s.get('schema', 'public')
            )

        return await asyncio.gather(*[execute_stmt(s) for s in statements])


# Example usage
if __name__ == '__main__':
    import asyncio

    async def main():
        async with AsyncPostgresClient(host='localhost', port=50061, user_id='test_user') as client:
            # Health check
            health = await client.health_check()
            print(f"Health: {health}")

            # Query
            tables = await client.list_tables()
            print(f"Tables: {tables}")

            # Concurrent queries
            results = await client.query_many_concurrent([
                {'sql': 'SELECT 1 as num'},
                {'sql': 'SELECT 2 as num'},
                {'sql': 'SELECT 3 as num'}
            ])
            print(f"Concurrent results: {results}")

    asyncio.run(main())
